# Taken from http://www.sqlalchemy.org/trac/wiki/UsageRecipes/Enum

# # The MIT License

# Copyright (c) <year> <copyright holders>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from plone.base.utils import safe_text
from sqlalchemy import types

import uuid


class UUID(types.TypeEngine):
    def get_col_spec(self):
        return "UUID"

    def bind_processor(self, dialect):
        def convert(value):
            if isinstance(value, str):
                return value
            elif isinstance(value, uuid.UUID):
                return str(value)
            raise TypeError

        return convert

    def result_processor(self, dialect):
        def convert(value):
            return value

        return convert

    def convert_result_value(self, value, dialect):
        return value

    def convert_bind_param(self, value, dialect):
        return value


class Enum(types.TypeDecorator):
    cache_ok = True
    impl = types.Unicode

    def __init__(self, values, empty_to_none=False, strict=False):
        """Emulate an Enum type.

        values:
           A list of valid values for this column
        empty_to_none:
           Optional, treat the empty string '' as None
        strict:
           Also insist that columns read from the database are in the
           list of valid values.  Note that, with strict=True, you won't
           be able to clean out bad data from the database through your
           code.
        """

        if values is None or len(values) == 0:
            raise TypeError("Enum requires a list of values")
        self.empty_to_none = empty_to_none
        self.strict = strict
        # Cast to tuple to ensure it can be hashed and so cached
        self.values = tuple(values)

        # The length of the string/unicode column should be the longest string
        # in values
        size = max([len(v) for v in values if v is not None])
        super().__init__(size)

    def process_bind_param(self, value, dialect):
        if self.empty_to_none and value == "":
            value = None
        if value not in self.values:
            raise ValueError('"%s" not in Enum.values' % value)
        if value is None:
            return None
        else:
            return safe_text(value)

    def process_result_value(self, value, dialect):
        if value is None:
            return value
        if self.strict and value not in self.values:
            raise ValueError('"%s" not in Enum.values' % value)
        return safe_text(value)


if __name__ == "__main__":
    from sqlalchemy import Column
    from sqlalchemy import Integer
    from sqlalchemy import MetaData
    from sqlalchemy import Table

    t = Table(
        "foo",
        MetaData("sqlite:///"),
        Column("id", Integer, primary_key=True),
        Column("e", Enum(["foobar", "baz", "quux", None])),
    )
    t.create()

    t.insert().execute(e="foobar")
    t.insert().execute(e="baz")
    t.insert().execute(e="quux")
    t.insert().execute(e=None)

    try:
        t.insert().execute(e="lala")
        assert False
    except AssertionError:
        pass

    print(list(t.select().execute()))



# Helper to assist with tuning the matomo/piwik tracking code for Euphorie
from euphorie.client.interfaces import IClientSkinLayer
from plone import api
from Products.Five import BrowserView
from zope.interface import alsoProvides

import json
import logging
import urllib.parse


logger = logging.getLogger("MatomoView")


class JSONTrackingVariablesView(BrowserView):
    """A helper that returns variables needed for tracking as json
    must be called on the tool, not on the session
    """

    def __call__(self):
        self.request.response.setHeader("Content-Type", "application/json")
        self.request.response.setHeader(
            "Cache-Control", "no-cache, no-store, must-revalidate"
        )
        self.request.response.setHeader("Pragma", "no-cache")
        self.request.response.setHeader("Expires", "0")
        alsoProvides(self.request, IClientSkinLayer)
        webhelpers = api.content.get_view("webhelpers", self.context, self.request)

        data = {
            "country_name": webhelpers.country_name,
            "sector_name": webhelpers.sector_name,
            "tool_name": webhelpers.tool_name,
            "language_code": webhelpers.language_code,
        }
        came_from = self.request.form.get("came_from", "")

        # Get the data from the came_from query string if not already set
        if came_from:
            # Parse query string from
            # came_from=/oira-tools/eu/eu-automation-of-tasks/eu-automation-of-tasks .
            path_elems = came_from.split("/")

            # Extract missing values from `came_from`
            if not data["country_name"] and len(path_elems) > 2:
                data["country_name"] = path_elems[2]
            if not data["sector_name"] and len(path_elems) > 3:
                data["sector_name"] = path_elems[3]
            if not data["tool_name"] and len(path_elems) > 4:
                data["tool_name"] = path_elems[4]

        return json.dumps(data, indent=2)


class MockTrackingView(BrowserView):
    """A simple tracking endpoint mimicking a piwik.php request"""

    def __call__(self):
        request = self.request
        method = request.get("REQUEST_METHOD", "GET")

        # Extract tracking parameters from GET or POST
        params = (
            request.form
            if method == "POST"
            else request.environ.get("QUERY_STRING", "")
        )
        parsed_params = urllib.parse.parse_qs(params)
        logger.info("\n--- Decoded Tracking Request ---\n")

        # Log the request details
        for key, value in parsed_params.items():
            logger.info(f"{key}: {', '.join(value)}")

        # Return a 1x1 transparent pixel to simulate tracking
        self.request.response.setHeader("Content-Type", "image/gif")
        self.request.response.setHeader(
            "Cache-Control", "no-cache, no-store, must-revalidate"
        )
        self.request.response.setHeader("Pragma", "no-cache")
        self.request.response.setHeader("Expires", "0")

        # Return a simple 1x1 pixel
        pixel = b"\x47\x49\x46\x38\x39\x61\x01\x00\x01\x00\x80\x00\x00\xff\xff\xff\x00\x00\x00\x21\xf9\x04\x01\x00\x00\x00\x00\x2c\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02\x4c\x01\x00\x3b"  # noqa: E501
        return pixel



from plone import api
from plone.app.vocabularies.terms import safe_simplevocabulary_from_values
from zope.interface import implementer
from zope.schema.interfaces import IContextSourceBinder


@implementer(IContextSourceBinder)
class RegistryValueVocabulary:
    def __init__(self, value_name):
        self.value_name = value_name

    def __len__(self):
        return len(self.values)

    @property
    def values(self):
        return tuple(
            filter(None, api.portal.get_registry_record(self.value_name, default=[]))
        )

    def __call__(self, context):
        return safe_simplevocabulary_from_values(self.values)



"""
Solution
--------

A standard Solution that a user can select for a particular risk.
"""

from .. import MessageFactory as _
from euphorie.content import utils
from plone.autoform import directives
from plone.base.utils import safe_text
from plone.dexterity.content import Item
from plone.indexer import indexer
from plone.supermodel import model
from zope import schema
from zope.i18n import translate
from zope.interface import implementer


class ISolution(model.Schema):
    """A standard solution for a risk.

    Risk questions can have standard solutions that can be applied in
    most environments.
    """

    description = schema.Text(
        title=_("label_module_description", "Description"),
        description=_(
            "help_module_description",
            default="Include any relevant information that may be "
            "helpful for the end-user.",
        ),
        required=True,
    )
    directives.order_after(description="title")

    action_plan = schema.Text(
        title=_(
            "label_measure_action_plan",
            default="General approach (to eliminate or reduce the risk)",
        ),
        description=_(
            "help_measure_action_plan",
            default="Describe your general approach to eliminate or (if "
            "the risk is not avoidable) reduce the risk.",
        ),
        required=False,
    )

    prevention_plan = schema.Text(
        title=_(
            "label_measure_prevention_plan",
            default="Specific action(s) required to implement this " "approach",
        ),
        description=_(
            "help_measure_prevention_plan",
            default="Describe the specific action(s) required to "
            "implement this approach (to eliminate or to reduce the risk).",
        ),
        required=False,
    )

    # This replaces action_plan and prevention_plan by concatenating the 2 fields.
    action = schema.Text(
        title=_(
            "label_measure_action",
            default=(
                "General approach (to eliminate or reduce the risk) + "
                "Specific action(s) required to implement this approach"
            ),
        ),
        description=_(
            "help_measure_action",
            default=(
                "Describe your general approach to eliminate or "
                "(if the risk is not avoidable) "
                "reduce the risk. + Describe the specific action(s) "
                "required to implement this approach "
                "(to eliminate or to reduce the risk)."
            ),
        ),
        required=True,
    )

    requirements = schema.Text(
        title=_(
            "label_measure_requirements",
            default="Level of expertise and/or requirements needed",
        ),
        description=_(
            "help_measure_requirements",
            default="Describe the level of expertise needed to implement "
            'the measure, for instance "common sense (no OSH knowledge '
            'required)", "no specific OSH expertise, but minimum OSH '
            "knowledge or training and/or consultation of OSH guidance "
            'required", or "OSH expert". You can also describe here '
            "any other additional requirement (if any).",
        ),
        required=False,
    )

    # show_in_identification = schema.Bool(
    #     title=_(
    #         "label_show_in_identification",
    #         default=u"Show this measure during Identification?"),
    #     description=_(
    #         "help_show_in_identification",
    #         default=u"Show this solution during the Identification phase as a "
    #         u"potential measure that is already in place? If the user selects "
    #         u"this measure during Identification, it will no longer be offered"
    #         u" in the Action Plan phase."),
    #     default=True,
    #     required=False,
    # )


@implementer(ISolution)
class Solution(Item):
    title = _("title_common_solution", default="Measure")
    prevention_plan = None
    requirements = None
    portal_type = "euphorie.solution"

    def Title(self):
        survey = utils.getSurvey(self)
        return safe_text(
            translate(
                Solution.title, context=self.REQUEST, target_language=survey.language
            )
        )


@indexer(ISolution)
def SearchableTextIndexer(obj):
    """Index the problem_description, question and solution_direction."""
    return " ".join(
        [
            obj.description,
            obj.action_plan or "",
            obj.prevention_plan or "",
            obj.requirements or "",
            obj.action or "",
        ]
    )



"""Empty message.

Revision ID: 28
Revises: 27
Create Date: 2020-05-28 12:52:00.626910
"""

from alembic import op
from euphorie.deployment.upgrade.utils import has_column

import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "28"
down_revision = "27"
branch_labels = None
depends_on = None


def upgrade():
    if not has_column("action_plan", "used_in_training"):
        op.add_column(
            "action_plan", sa.Column("used_in_training", sa.Boolean(), nullable=True)
        )
        op.create_index(
            op.f("ix_action_plan_used_in_training"),
            "action_plan",
            ["used_in_training"],
            unique=False,
        )
        op.execute(
            "UPDATE action_plan set used_in_training=True "
            "where plan_type in ('in_place_standard', 'in_place_custom')"
        )


def downgrade():
    op.drop_index(op.f("ix_action_plan_used_in_training"), table_name="action_plan")
    op.drop_column("action_plan", "used_in_training")



from Acquisition import aq_inner
from euphorie.client import CONDITIONS_VERSION
from euphorie.client.model import get_current_account
from plone import api
from Products.Five import BrowserView

import logging


log = logging.getLogger(__name__)


def approvedTermsAndConditions(account=None):
    if account is None:
        account = get_current_account()
    return account.tc_approved is not None and account.tc_approved == CONDITIONS_VERSION


class TermsAndConditions(BrowserView):
    def terms_changed(self):
        return getattr(self.account, "tc_approved", None) is not None

    def __call__(self):
        webhelpers = api.content.get_view(
            name="webhelpers", context=self.context, request=self.request
        )
        self.came_from = webhelpers.get_came_from(
            default=aq_inner(self.context).absolute_url()
        )

        self.account = get_current_account()
        if self.request.environ["REQUEST_METHOD"] == "POST":
            self.account.tc_approved = CONDITIONS_VERSION

            self.request.response.redirect(self.came_from)
        return self.index()



from euphorie.client import model
from euphorie.client.interfaces import IClientSkinLayer
from euphorie.client.tests.utils import addAccount
from euphorie.client.tests.utils import addSurvey
from euphorie.content.tests.utils import BASIC_SURVEY
from euphorie.testing import EuphorieIntegrationTestCase
from plone import api
from random import seed
from zExceptions import Unauthorized
from zope.interface import alsoProvides


class TestTrainingQuestions(EuphorieIntegrationTestCase):
    """There is a possibility to add training questions in a survey and
    evaluate the user comprehension of the training."""

    def setUp(self):
        super().setUp()
        with api.env.adopt_user("admin"):
            addSurvey(self.portal, BASIC_SURVEY)
        self.account = addAccount(password="secret")
        self.survey = self.portal.client.nl.ict["software-development"]
        alsoProvides(self.survey.REQUEST, IClientSkinLayer)
        api.portal.set_registry_record("euphorie.use_training_module", True)
        survey_session = model.SurveySession(
            title="Dummy session",
            zodb_path="nl/ict/software-development",
            account=self.account,
            company=model.Company(country="nl", employees="1-9", referer="other"),
        )
        model.Session.add(survey_session)
        seed(a="test_training_questions")

    def _create_questions(self):
        with api.env.adopt_user("admin"):
            api.content.create(
                id="question-1",
                container=self.survey,
                type="euphorie.training_question",
                title="Why?",
                right_answer="Because!",
                wrong_answer_1="White",
                wrong_answer_2="Black",
            )
            api.content.create(
                id="question-2",
                container=self.survey,
                type="euphorie.training_question",
                title="Life on Mars?",
                right_answer="Probably not",
                wrong_answer_1="In the weekends",
                wrong_answer_2="42",
            )
            api.content.create(
                id="question-3",
                container=self.survey,
                type="euphorie.training_question",
                title="Who are you?",
                right_answer="The Who 🞋",
                wrong_answer_1="Annie Lennox",
                wrong_answer_2="David Bowie",
            )

    def test_training_enabled(self):
        traversed_session = self.survey.restrictedTraverse("++session++1")
        self.portal.sectors.nl.enable_web_training = False
        with api.env.adopt_user(user=self.account):
            with self._get_view(
                "start", traversed_session, self.request.clone()
            ) as view:
                self.assertNotIn(
                    "Training",
                    view(),
                    msg="Training disabled on country but link visible",
                )

            self.survey.enable_web_training = True
            self.survey.enable_test_questions = True
            self._create_questions()
            with self._get_view(
                "start", traversed_session, self.request.clone()
            ) as view:
                self.assertNotIn(
                    "Training",
                    view(),
                    msg="Training disabled on country but link visible",
                )

            self.portal.sectors.nl.enable_web_training = True
            with self._get_view(
                "start", traversed_session, self.request.clone()
            ) as view:
                self.assertIn(
                    "Training", view(), msg="Training enabled but link invisible"
                )

    def test_num_training_questions(self):
        traversed_session = self.survey.restrictedTraverse("++session++1")
        seed(a="test_num_training_questions")
        with api.env.adopt_user(user=self.account):
            self.survey.enable_web_training = True
            self.survey.enable_test_questions = True
            self.survey.num_training_questions = 2
            self._create_questions()

            with self._get_view(
                "slide_question_intro", traversed_session, self.request.clone()
            ) as view:
                self.assertEqual(len(view.question_ids), 2)
                self.assertListEqual(
                    list(view.question_ids),
                    ["question-2", "question-3"],
                )
                self.assertEqual(
                    view.first_question_url(),
                    "http://nohost/plone/client/nl/ict/software-development/"
                    "++session++1/@@slide_question/question-2",
                )

    def test_training_questions(self):
        traversed_session = self.survey.restrictedTraverse("++session++1")
        with api.env.adopt_user(user=self.account):
            with self._get_view(
                "training", traversed_session, self.request.clone()
            ) as view:
                # No training if the survey has not enabled it
                self.assertEqual(view.question_intro_url, "")
            self.survey.enable_web_training = True
            self.survey.enable_test_questions = True
            with self._get_view(
                "training", traversed_session, self.request.clone()
            ) as view:
                # Jump direcly to the success page if there are no questions
                self.assertEqual(
                    view.question_intro_url,
                    "http://nohost/plone/client/nl/ict/software-development/++session++1/@@slide_question_success",  # noqa: E501
                )
            self._create_questions()
            with self._get_view(
                "training", traversed_session, self.request.clone()
            ) as view:
                self.assertEqual(
                    view.question_intro_url,
                    "http://nohost/plone/client/nl/ict/software-development/++session++1/@@slide_question_intro",  # noqa: E501
                )

            # The slide_question_intro is just a presentation of the questionaire
            with self._get_view(
                "slide_question_intro", traversed_session, self.request.clone()
            ) as view:
                self.assertEqual(len(view.question_ids), 3)
                self.assertListEqual(
                    list(view.question_ids),
                    ["question-3", "question-2", "question-1"],
                )
                self.assertEqual(
                    view.first_question_url(),
                    "http://nohost/plone/client/nl/ict/software-development/++session++1/@@slide_question/question-3",  # noqa: E501
                )

            # There is a certificate view but we cannot see it
            # if we have not started the training
            with self._get_view(
                "training-certificate", traversed_session, self.request.clone()
            ) as view:
                with self.assertRaises(Unauthorized):
                    view()

            # We can view the slides only if we have completed the previous ones
            with self._get_view(
                "slide_question", traversed_session, self.request.clone()
            ) as view:
                # emulate traversing and test some method/properties
                view.question_id = "question-3"
                self.assertEqual(view.progress, "1/3")
                self.assertEqual(view.question.title, "Who are you?")
                self.assertIsNone(view.previous_question_id)
                self.assertEqual(
                    self.survey[view.next_question_id].title, "Life on Mars?"
                )
                self.assertEqual(
                    view.next_url,
                    "http://nohost/plone/client/nl/ict/software-development/++session++1/@@slide_question/question-2",  # noqa: E501
                )

                view.question_id = "question-2"
                view.request.__annotations__.clear()
                self.assertEqual(view.progress, "2/3")
                self.assertEqual(view.question.title, "Life on Mars?")
                self.assertEqual(
                    self.survey[view.previous_question_id].title, "Who are you?"
                )
                self.assertEqual(self.survey[view.next_question_id].title, "Why?")
                self.assertEqual(
                    view.next_url,
                    "http://nohost/plone/client/nl/ict/software-development/++session++1/@@slide_question/question-1",  # noqa: E501
                )

                view.question_id = "question-1"
                view.request.__annotations__.clear()
                self.assertEqual(view.progress, "3/3")
                self.assertEqual(view.question.title, "Why?")
                self.assertEqual(
                    self.survey[view.previous_question_id].title, "Life on Mars?"
                )
                self.assertIsNone(view.next_question_id)
                self.assertEqual(
                    view.next_url,
                    "http://nohost/plone/client/nl/ict/software-development/++session++1/@@slide_question_try_again",  # noqa: E501
                )

                # Check validation so that we do not call the wrong slide
                view.question_id = "question-3"
                view.request.__annotations__.clear()
                view.validate()

                view.question_id = "question-2"
                view.request.__annotations__.clear()
                with self.assertRaises(Unauthorized):
                    view.validate()

                view.question_id = "question-1"
                view.request.__annotations__.clear()
                with self.assertRaises(Unauthorized):
                    view.validate()

                # Try to give some answer
                view.question_id = "question-3"
                view.request.__annotations__.clear()
                view.request.method = "POST"
                view.request.form["answer"] = "The Who 🞋"
                self.assertEqual(view(), view.next_url)
                self.assertEqual(
                    view.get_or_create_training().answers,
                    '{"question-3": true, "question-2": null, "question-1": null}',
                )
                self.assertEqual(view.get_or_create_training().status, "in_progress")

                # Now we can see the first two slides
                view.question_id = "question-3"
                view.request.__annotations__.clear()
                self.assertIsNone(view.validate())

                view.question_id = "question-2"
                view.request.__annotations__.clear()
                self.assertIsNone(view.validate())

                # but not yet the last...
                view.question_id = "question-1"
                view.request.__annotations__.clear()
                with self.assertRaises(Unauthorized):
                    view.validate()

                # There is a certificate view but we cannot see it
                # if we have not completed the training
                with self._get_view(
                    "training-certificate", traversed_session, self.request.clone()
                ) as certificate:
                    with self.assertRaises(Unauthorized):
                        certificate()

                # We answer wrongly the second answer
                view.question_id = "question-2"
                view.request.__annotations__.clear()
                view.request.form["answer"] = "Foo"
                self.assertEqual(view(), view.next_url)
                self.assertIn(
                    '"question-2": false', view.get_or_create_training().answers
                )

                # We answer correctly the third answer
                view.question_id = "question-1"
                view.request.__annotations__.clear()
                view.request.form["answer"] = "Because!"
                self.assertEqual(view(), view.next_url)
                self.assertIn(
                    '"question-1": true', view.get_or_create_training().answers
                )
                self.assertEqual(view.get_or_create_training().status, "failed")

            with self._get_view(
                "slide_question_try_again", traversed_session, self.request.clone()
            ) as view:
                self.assertListEqual(view.failed_questions, ["Life on Mars?"])

            # There is a certificate view but we cannot see it
            # if we have not completed the training successfully
            with self._get_view(
                "training-certificate", traversed_session, self.request.clone()
            ) as view:
                with self.assertRaises(Unauthorized):
                    view()

            # We try again
            with self._get_view(
                "slide_question", traversed_session, self.request.clone()
            ) as view:
                view.question_id = "question-3"
                view.request.method = "POST"
                view.request.form["answer"] = "The Who 🞋"
                view()

                # Trying again resets resets the answers
                self.assertEqual(
                    view.get_or_create_training().answers,
                    '{"question-3": true, "question-2": null, "question-1": null}',
                )
                view.question_id = "question-2"
                view.request.__annotations__.clear()
                view.request.form["answer"] = "Probably not"
                view()

                view.question_id = "question-1"
                view.request.__annotations__.clear()
                view.request.form["answer"] = "Because!"
                view()
                self.assertEqual(view(), view.next_url)
                self.assertEqual(
                    view.next_url,
                    "http://nohost/plone/client/nl/ict/software-development/++session++1/@@slide_question_success",  # noqa: E501
                )

            # Now we can view the certificate!
            with self._get_view(
                "training-certificate", traversed_session, self.request.clone()
            ) as view:
                view()



from zope.interface import implementer


try:
    from plone.base.interfaces.installable import INonInstallable
except ImportError:
    from Products.CMFPlone.interfaces import INonInstallable


@implementer(INonInstallable)
class HideEuphorieProducts:
    def getNonInstallableProfiles(self):
        return []

    def getNonInstallableProducts(self):
        return ["euphorie.content", "euphorie.client"]



from euphorie.client import model
from euphorie.client.config import LOCKING_ACTIONS
from plone import api
from plone.memoize.view import memoize
from plone.memoize.view import memoize_contextless
from Products.Five import BrowserView
from zExceptions import Unauthorized


class LockingMenu(BrowserView):
    lock_actions = LOCKING_ACTIONS

    @property
    @memoize
    def webhelpers(self):
        return api.content.get_view("webhelpers", self.context, self.request)

    def redirect(self):
        target = self.request.get("HTTP_REFERER") or self.context.absolute_url()
        return self.request.response.redirect(target)

    @property
    @memoize_contextless
    def portal(self):
        """The currently authenticated account."""
        return api.portal.get()

    @property
    def is_locked(self):
        """Return whether the session is locked."""
        return self.context.session.is_locked

    @property
    def is_validated(self):
        """Return whether the session is validated."""
        return self.context.session.is_validated

    def show_actions(self):
        """Return whether we should show the actions in the menu."""
        if self.is_locked:
            return self.webhelpers.can_unlock_session
        return self.webhelpers.can_lock_session

    @property
    def state(self):
        """Return the state of the session."""
        if self.is_validated:
            return "validated"
        if self.is_locked:
            return "locked"
        return "unlocked"

    def create_event(self, action):
        event = model.SessionEvent(
            account_id=self.webhelpers.current_account.id,
            session_id=self.context.session.id,
            action=action,
        )
        model.Session.add(event)

    def refresh_lock(self):
        """Reset the session date to now."""
        if not self.webhelpers.can_lock_session:
            raise Unauthorized()
        self.create_event("lock_refresh")
        return self.redirect()

    def set_lock(self):
        """Set the session date to now."""
        if not self.webhelpers.can_lock_session:
            raise Unauthorized()
        self.create_event("lock_set")
        return self.redirect()

    def unset_lock(self):
        """Unset the session date."""
        if not self.webhelpers.can_unlock_session:
            raise Unauthorized()
        self.create_event("lock_unset")
        return self.redirect()



from euphorie.client.interfaces import INotificationCategory
from euphorie.client.interfaces import INTERVAL_DAILY
from plone import api
from plone.protect.interfaces import IDisableCSRFProtection
from Products.Five import BrowserView
from zope.component import getAdapters
from zope.interface import alsoProvides


class SendNotificationsDaily(BrowserView):
    def do_send(self):
        if not api.portal.get_registry_record(
            "euphorie.notifications__enabled", default=False
        ):
            return
        notifications = getAdapters((self.context, self.request), INotificationCategory)
        for notification in notifications:
            if not notification[1].available:
                continue
            if not notification[1].interval == INTERVAL_DAILY:
                continue
            notification[1].notify()

    def __call__(self):
        alsoProvides(self.request, IDisableCSRFProtection)
        self.do_send()



from datetime import datetime
from datetime import timedelta
from euphorie.client import model
from euphorie.client.browser.company import Company
from euphorie.client.tests.utils import addAccount
from euphorie.client.tests.utils import addSurvey
from euphorie.content.tests.utils import BASIC_SURVEY
from euphorie.testing import EuphorieIntegrationTestCase
from plone import api
from unittest import mock


class TestCompanyViews(EuphorieIntegrationTestCase):
    """"""

    @mock.patch.object(Company, "applyChanges")
    @mock.patch.object(Company, "render")
    def test_company_timestamp(self, mock_render, mock_applyChanges):
        with api.env.adopt_user("admin"):
            survey = addSurvey(self.portal, BASIC_SURVEY)
        account = addAccount(password="secret")
        survey_session = model.SurveySession(
            id=456,
            title="Dummy session",
            created=datetime(2021, 4, 9, 9, 11, 31),
            modified=datetime(2021, 4, 9, 9, 11, 52),
            zodb_path="nl/ict/software-development",
            account=account,
        )
        model.Session.add(survey_session)
        survey = self.portal.client.nl.ict["software-development"]

        session_id = "++session++%d" % survey_session.id
        traversed_survey_session = survey.restrictedTraverse(session_id)

        with api.env.adopt_user(user=survey_session.account):
            with self._get_view(
                "report_company", traversed_survey_session, survey_session
            ) as view:
                view.request.form = {
                    "form.widgets.conductor": "staff",
                    "form.widgets.country": "nl",
                    "form.widgets.employees": "10-49",
                    "form.widgets.referer": "health-safety-experts",
                    "form.widgets.workers_participated": False,
                    "form.widgets.needs_met": True,
                    "form.widgets.recommend_tool": True,
                    "form.buttons.next": "",
                }
                view()
                timestamp = mock_applyChanges.call_args[0][0]["timestamp"]
                self.assertLess(datetime.now() - timestamp, timedelta(seconds=3))

                del view.request.form["form.buttons.next"]
                view.request.form["form.buttons.previous"] = ""
                view()
                self.assertEqual(mock_applyChanges.call_count, 2)
                timestamp = mock_applyChanges.call_args[0][0]["timestamp"]
                self.assertLess(datetime.now() - timestamp, timedelta(seconds=3))



from ftw.upgrade import UpgradeStep
from plone import api


class SetPasswordExpirationTimeout(UpgradeStep):
    """Set password expiration timeout."""

    def __call__(self):
        ppr = api.portal.get_tool("portal_password_reset")
        ppr.setExpirationTimeout(0.5)



from euphorie.client import model
from euphorie.client.browser.webhelpers import WebHelpers
from euphorie.client.tests.utils import addAccount
from euphorie.client.tests.utils import addSurvey
from euphorie.content.tests.utils import BASIC_SURVEY
from euphorie.testing import EuphorieIntegrationTestCase
from plone import api
from plone.app.testing.interfaces import SITE_OWNER_NAME
from unittest import mock
from unittest import TestCase


class TestWebhelpers(EuphorieIntegrationTestCase):
    def test_get_sessions_query_anonymous(self):
        with self._get_view("webhelpers", self.portal.client) as view:
            # anonymous does not see anything
            self.assertIn("WHERE 0 = 1", str(view.get_sessions_query()))

    def _get_query_filters(self, query):
        """Return the filters of a SQLAlchemy query."""
        return str(query).partition("\nFROM session \n")[-1]

    def test_content_country_object(self):
        with api.env.adopt_user(SITE_OWNER_NAME):
            content_country = api.content.create(
                container=self.portal.sectors, type="euphorie.country", id="eu"
            )
            client_country = api.content.create(
                container=self.portal.client, type="euphorie.clientcountry", id="eu"
            )

        with self._get_view("webhelpers", self.portal.client.eu) as view:
            self.assertNotEqual(view.content_country_obj, client_country)
            self.assertEqual(view.content_country_obj, content_country)

    def test_get_sessions_query_authenticated(self):
        account = addAccount(password="secret")
        with api.env.adopt_user("admin"):
            eu = api.content.create(
                type="euphorie.clientcountry", id="eu", container=self.portal.client
            )
            eusector = api.content.create(
                type="euphorie.clientsector", id="eusector", container=eu
            )
            api.content.create(
                type="euphorie.survey", id="eusurvey", container=eusector
            )
        with api.env.adopt_user(user=account):
            # Check with no parameter
            with self._get_view("webhelpers", self.portal.client) as view:
                self.assertEqual(
                    self._get_query_filters(view.get_sessions_query()),
                    (
                        "WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        "session.account_id = ? AND "
                        "(session.archived >= ? OR session.archived IS NULL) "
                        "ORDER BY session.modified DESC, session.title"
                    ),
                )

                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(include_archived=True)
                    ),
                    (
                        "WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        "session.account_id = ? "
                        "ORDER BY session.modified DESC, session.title"
                    ),
                )
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(searchable_text="foo")
                    ),
                    (
                        "WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        "session.account_id = ? AND "
                        "(session.archived >= ? OR session.archived IS NULL) AND "
                        "lower(session.title) LIKE lower(?) "
                        "ORDER BY session.modified DESC, session.title"
                    ),
                )
                foo_group = model.Group(group_id="foo")
                account.group = foo_group
                model.Session.flush()
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(filter_by_group=True)
                    ),
                    (
                        "WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        "session.account_id = ? AND "
                        "session.group_id = ? AND "
                        "(session.archived >= ? OR session.archived IS NULL) "
                        "ORDER BY session.modified DESC, session.title"
                    ),
                )
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(filter_by_account=False)
                    ),
                    (
                        "WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        "(session.archived >= ? OR session.archived IS NULL) "
                        "ORDER BY session.modified DESC, session.title"
                    ),
                )
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(
                            filter_by_account=False, filter_by_group=True
                        )
                    ),
                    (
                        "WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        "session.group_id = ? AND "
                        "(session.archived >= ? OR session.archived IS NULL) "
                        "ORDER BY session.modified DESC, session.title"
                    ),
                )
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(
                            filter_by_account=False, filter_by_group=foo_group
                        )
                    ),
                    (
                        "WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        "session.group_id = ? AND "
                        "(session.archived >= ? OR session.archived IS NULL) "
                        "ORDER BY session.modified DESC, session.title"
                    ),
                )
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(
                            filter_by_account=False, filter_by_group=[foo_group]
                        )
                    ),
                    (
                        "WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        "session.group_id = ? AND "
                        "(session.archived >= ? OR session.archived IS NULL) "
                        "ORDER BY session.modified DESC, session.title"
                    ),
                )
                bar_group = model.Group(group_id="bar")
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(
                            filter_by_account=False,
                            filter_by_group=[foo_group, bar_group],
                        )
                    ),
                    (
                        "WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        "session.group_id IN (__[POSTCOMPILE_group_id_1]) AND "
                        "(session.archived >= ? OR session.archived IS NULL) "
                        "ORDER BY session.modified DESC, session.title"
                    ),
                )

    def test_get_sessions_query_authenticated_with_organization(self):
        account1 = addAccount("foo", password="secret")
        account2 = addAccount("bar", password="secret")
        with api.env.adopt_user("admin"):
            eu = api.content.create(
                type="euphorie.clientcountry", id="eu", container=self.portal.client
            )
            eusector = api.content.create(
                type="euphorie.clientsector", id="eusector", container=eu
            )
            api.content.create(
                type="euphorie.survey", id="eusurvey", container=eusector
            )

        # Make account2 a member of account1 organization
        model.Session.add(
            model.OrganisationMembership(owner_id=account1.id, member_id=account2.id)
        )
        model.Session.flush()

        # For account1 the filter should be unchanged
        # (he sees only the sessions with its own id)
        session_filter = "session.account_id = ?"
        with api.env.adopt_user(user=account1):
            # Check with no parameter
            with self._get_view("webhelpers", self.portal.client) as view:
                self.assertEqual(
                    self._get_query_filters(view.get_sessions_query()),
                    (
                        f"WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        f"{session_filter} AND "
                        f"(session.archived >= ? OR session.archived IS NULL) "
                        f"ORDER BY session.modified DESC, session.title"
                    ),
                )

                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(include_archived=True)
                    ),
                    (
                        f"WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        f"{session_filter} "
                        f"ORDER BY session.modified DESC, session.title"
                    ),
                )
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(searchable_text="foo")
                    ),
                    (
                        f"WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        f"{session_filter} AND "
                        f"(session.archived >= ? OR session.archived IS NULL) AND "
                        f"lower(session.title) LIKE lower(?) "
                        f"ORDER BY session.modified DESC, session.title"
                    ),
                )
                foo_group = model.Group(group_id="foo")
                account1.group = foo_group
                model.Session.flush()
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(filter_by_group=True)
                    ),
                    (
                        f"WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        f"{session_filter} AND "
                        f"session.group_id = ? AND "
                        f"(session.archived >= ? OR session.archived IS NULL) "
                        f"ORDER BY session.modified DESC, session.title"
                    ),
                )
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(filter_by_account=False)
                    ),
                    (
                        "WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        "(session.archived >= ? OR session.archived IS NULL) "
                        "ORDER BY session.modified DESC, session.title"
                    ),
                )
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(
                            filter_by_account=False, filter_by_group=True
                        )
                    ),
                    (
                        "WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        "session.group_id = ? AND "
                        "(session.archived >= ? OR session.archived IS NULL) "
                        "ORDER BY session.modified DESC, session.title"
                    ),
                )
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(
                            filter_by_account=False, filter_by_group=foo_group
                        )
                    ),
                    (
                        "WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        "session.group_id = ? AND "
                        "(session.archived >= ? OR session.archived IS NULL) "
                        "ORDER BY session.modified DESC, session.title"
                    ),
                )
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(
                            filter_by_account=False, filter_by_group=[foo_group]
                        )
                    ),
                    (
                        "WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        "session.group_id = ? AND "
                        "(session.archived >= ? OR session.archived IS NULL) "
                        "ORDER BY session.modified DESC, session.title"
                    ),
                )
                bar_group = model.Group(group_id="bar")
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(
                            filter_by_account=False,
                            filter_by_group=[foo_group, bar_group],
                        )
                    ),
                    (
                        "WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        "session.group_id IN (__[POSTCOMPILE_group_id_1]) AND "
                        "(session.archived >= ? OR session.archived IS NULL) "
                        "ORDER BY session.modified DESC, session.title"
                    ),
                )

        # For account2 the filter is changed to include also the sessions of the
        # other organization members
        session_filter = "session.account_id IN (__[POSTCOMPILE_account_id_1])"
        with api.env.adopt_user(user=account2):
            # Check with no parameter
            with self._get_view("webhelpers", self.portal.client) as view:
                self.assertEqual(
                    self._get_query_filters(view.get_sessions_query()),
                    (
                        f"WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        f"{session_filter} AND "
                        f"(session.archived >= ? OR session.archived IS NULL) "
                        f"ORDER BY session.modified DESC, session.title"
                    ),
                )
                view.get_sessions_query(include_archived=True)
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(include_archived=True)
                    ),
                    (
                        f"WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        f"{session_filter} "
                        f"ORDER BY session.modified DESC, session.title"
                    ),
                )
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(searchable_text="foo")
                    ),
                    (
                        f"WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        f"{session_filter} AND "
                        f"(session.archived >= ? OR session.archived IS NULL) AND "
                        f"lower(session.title) LIKE lower(?) "
                        f"ORDER BY session.modified DESC, session.title"
                    ),
                )
                baz_group = model.Group(group_id="baz")
                account2.group = baz_group
                model.Session.flush()
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(filter_by_group=True)
                    ),
                    (
                        f"WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        f"{session_filter} AND "
                        f"session.group_id = ? AND "
                        f"(session.archived >= ? OR session.archived IS NULL) "
                        f"ORDER BY session.modified DESC, session.title"
                    ),
                )
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(filter_by_account=False)
                    ),
                    (
                        "WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        "(session.archived >= ? OR session.archived IS NULL) "
                        "ORDER BY session.modified DESC, session.title"
                    ),
                )
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(
                            filter_by_account=False, filter_by_group=True
                        )
                    ),
                    (
                        "WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        "session.group_id = ? AND "
                        "(session.archived >= ? OR session.archived IS NULL) "
                        "ORDER BY session.modified DESC, session.title"
                    ),
                )
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(
                            filter_by_account=False, filter_by_group=foo_group
                        )
                    ),
                    (
                        "WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        "session.group_id = ? AND "
                        "(session.archived >= ? OR session.archived IS NULL) "
                        "ORDER BY session.modified DESC, session.title"
                    ),
                )
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(
                            filter_by_account=False, filter_by_group=[foo_group]
                        )
                    ),
                    (
                        "WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        "session.group_id = ? AND "
                        "(session.archived >= ? OR session.archived IS NULL) "
                        "ORDER BY session.modified DESC, session.title"
                    ),
                )
                fiz_group = model.Group(group_id="fiz")
                self.assertEqual(
                    self._get_query_filters(
                        view.get_sessions_query(
                            filter_by_account=False,
                            filter_by_group=[baz_group, fiz_group],
                        )
                    ),
                    (
                        "WHERE session.zodb_path IN (__[POSTCOMPILE_zodb_path_1]) AND "
                        "session.group_id IN (__[POSTCOMPILE_group_id_1]) AND "
                        "(session.archived >= ? OR session.archived IS NULL) "
                        "ORDER BY session.modified DESC, session.title"
                    ),
                )

    def test_can_view_session(self):
        # Setup a bunch of accounts and sessions
        account1 = addAccount("foo", password="secret")
        account2 = addAccount("bar", password="secret")
        account3 = addAccount("baz", password="secret")

        with api.env.adopt_user("admin"):
            addSurvey(self.portal, BASIC_SURVEY)

        survey_session1 = model.SurveySession(
            id=1,
            title="Dummy session",
            zodb_path="nl/ict/software-development",
            account=account1,
        )
        model.Session.add(survey_session1)

        survey_session2 = model.SurveySession(
            id=2,
            title="Dummy session",
            zodb_path="nl/ict/software-development",
            account=account2,
        )
        model.Session.add(survey_session2)

        country = self.portal.client.nl
        traversed_session1 = country.ict["software-development"].restrictedTraverse(
            "++session++1"
        )
        traversed_session2 = country.ict["software-development"].restrictedTraverse(
            "++session++2"
        )

        # account1 can view its own session
        with api.env.adopt_user(user=account1):
            with self._get_view("webhelpers", traversed_session1) as view:
                self.assertTrue(view.can_view_session)

        # account2 cannot view
        with api.env.adopt_user(user=account2):
            with self._get_view("webhelpers", traversed_session1) as view:
                self.assertFalse(view.can_view_session)

        # We now make account 2 a member of account 1 organization
        model.Session.add(
            model.OrganisationMembership(owner_id=account1.id, member_id=account2.id)
        )

        # now account2 can view
        with api.env.adopt_user(user=account2):
            with self._get_view("webhelpers", traversed_session1) as view:
                self.assertTrue(view.can_view_session)

        # of course account1 cannot view account2's session
        with api.env.adopt_user(user=account1):
            with self._get_view("webhelpers", traversed_session2) as view:
                self.assertFalse(view.can_view_session)

        # account3 still cannot
        with api.env.adopt_user(user=account3):
            with self._get_view("webhelpers", traversed_session1) as view:
                self.assertFalse(view.can_view_session)

    def test_is_survey(self):
        """Test if webhelper's context is within a survey."""

        # Setup basic content.

        account = addAccount("foo", password="secret")

        with api.env.adopt_user("admin"):
            addSurvey(self.portal, BASIC_SURVEY)

        survey_session = model.SurveySession(
            id=1,
            title="Dummy session",
            zodb_path="nl/ict/software-development",
            account=account,
        )
        model.Session.add(survey_session)

        traversed_session = self.portal.client.nl.ict[
            "software-development"
        ].restrictedTraverse("++session++1")

        # Tests

        with self._get_view("webhelpers", self.portal) as view:
            self.assertFalse(view.is_survey)

        with self._get_view("webhelpers", self.portal.client) as view:
            self.assertFalse(view.is_survey)

        with self._get_view("webhelpers", self.portal.client.nl) as view:
            self.assertFalse(view.is_survey)

        with self._get_view("webhelpers", self.portal.client.nl.ict) as view:
            self.assertFalse(view.is_survey)

        with self._get_view(
            "webhelpers", self.portal.client.nl.ict["software-development"]
        ) as view:
            self.assertTrue(view.is_survey)

        with self._get_view("webhelpers", traversed_session) as view:
            self.assertTrue(view.is_survey)

    def test_survey_tree_data(self):
        """Test the survey navigation tree data."""

        # Setup basic content.

        account = addAccount("foo", password="secret")

        with api.env.adopt_user("admin"):
            addSurvey(self.portal, BASIC_SURVEY)

        survey_session = model.SurveySession(
            id=1,
            title="Dummy session",
            zodb_path="nl/ict/software-development",
            account=account,
        )
        model.Session.add(survey_session)

        traversed_session = self.portal.client.nl.ict[
            "software-development"
        ].restrictedTraverse("++session++1")

        # Tests

        with mock.patch(
            "euphorie.client.browser.webhelpers.WebHelpers.is_new_session",
            new_callable=mock.PropertyMock,
        ) as is_new_session:
            is_new_session.return_value = False
            with mock.patch(
                "euphorie.client.browser.webhelpers.WebHelpers.get_phase",
                return_value="preparation",
            ):
                is_new_session.return_value = False
                with api.env.adopt_user(user=account):
                    with self._get_view("webhelpers", traversed_session) as view:
                        nav_tree = view.survey_tree_data

                        self.assertEqual(nav_tree[0]["id"], "step-1")
                        self.assertEqual(nav_tree[1]["id"], "step-2")
                        self.assertEqual(nav_tree[2]["id"], "step-4")
                        self.assertEqual(nav_tree[3]["id"], "step-5")
                        self.assertEqual(nav_tree[4]["id"], "status")

                        self.assertTrue("@@start" in nav_tree[0]["href"])
                        self.assertTrue("@@identification" in nav_tree[1]["href"])
                        self.assertTrue("@@actionplan" in nav_tree[2]["href"])
                        self.assertTrue("@@report" in nav_tree[3]["href"])
                        self.assertTrue("@@status" in nav_tree[4]["href"])

                        self.assertEqual(nav_tree[0]["active"], True)
                        self.assertEqual(nav_tree[1]["active"], False)
                        self.assertEqual(nav_tree[2]["active"], False)
                        self.assertEqual(nav_tree[3]["active"], False)
                        self.assertEqual(nav_tree[4]["active"], False)

                        self.assertEqual(nav_tree[0]["disabled"], False)
                        self.assertEqual(nav_tree[1]["disabled"], False)
                        self.assertEqual(nav_tree[2]["disabled"], False)
                        self.assertEqual(nav_tree[3]["disabled"], False)
                        self.assertEqual(nav_tree[4]["disabled"], False)

    def test_survey_tree_data__archived__training(self):
        """Test the survey navigation tree data when survey is archived and the
        training module active.

        The report, training and status navigation items should not be
        disabled.
        """

        # Setup basic content.

        account = addAccount("foo", password="secret")

        with api.env.adopt_user("admin"):
            addSurvey(self.portal, BASIC_SURVEY)

        survey_session = model.SurveySession(
            id=1,
            title="Dummy session",
            zodb_path="nl/ict/software-development",
            account=account,
        )
        model.Session.add(survey_session)

        traversed_session = self.portal.client.nl.ict[
            "software-development"
        ].restrictedTraverse("++session++1")

        # Tests

        with mock.patch(
            "euphorie.client.model.SurveySession.is_archived",
            new_callable=mock.PropertyMock,
        ) as is_archived:
            is_archived.return_value = True
            with mock.patch(
                "euphorie.client.browser.webhelpers.WebHelpers.use_training_module",
                new_callable=mock.PropertyMock,
            ) as use_training_module:
                use_training_module.return_value = True
                with mock.patch(
                    "euphorie.client.browser.webhelpers.WebHelpers.is_new_session",
                    new_callable=mock.PropertyMock,
                ) as is_new_session:
                    is_new_session.return_value = False
                    with mock.patch(
                        "euphorie.client.browser.webhelpers.WebHelpers.get_phase",
                        return_value="preparation",
                    ):
                        is_new_session.return_value = False
                        with api.env.adopt_user(user=account):
                            with self._get_view(
                                "webhelpers", traversed_session
                            ) as view:
                                nav_tree = view.survey_tree_data

                                self.assertEqual(nav_tree[0]["id"], "step-1")
                                self.assertEqual(nav_tree[1]["id"], "step-2")
                                self.assertEqual(nav_tree[2]["id"], "step-4")
                                self.assertEqual(nav_tree[3]["id"], "step-5")
                                self.assertEqual(nav_tree[4]["id"], "step-6")
                                self.assertEqual(nav_tree[5]["id"], "status")

                                self.assertTrue("@@start" in nav_tree[0]["href"])
                                self.assertTrue(
                                    "@@identification" in nav_tree[1]["href"]
                                )
                                self.assertTrue("@@actionplan" in nav_tree[2]["href"])
                                self.assertTrue("@@report" in nav_tree[3]["href"])
                                self.assertTrue("@@training" in nav_tree[4]["href"])
                                self.assertTrue("@@status" in nav_tree[5]["href"])

                                self.assertEqual(nav_tree[0]["active"], True)
                                self.assertEqual(nav_tree[1]["active"], False)
                                self.assertEqual(nav_tree[2]["active"], False)
                                self.assertEqual(nav_tree[3]["active"], False)
                                self.assertEqual(nav_tree[4]["active"], False)
                                self.assertEqual(nav_tree[5]["active"], False)

                                self.assertEqual(nav_tree[0]["disabled"], False)
                                self.assertEqual(nav_tree[1]["disabled"], True)
                                self.assertEqual(nav_tree[2]["disabled"], True)
                                self.assertEqual(nav_tree[3]["disabled"], False)
                                self.assertEqual(nav_tree[4]["disabled"], False)
                                self.assertEqual(nav_tree[5]["disabled"], False)

    def test_survey_tree_data__consultancy(self):
        """Test the survey navigation tree data when consultancy is active."""

        # Setup basic content.

        account = addAccount("foo", password="secret")

        with api.env.adopt_user("admin"):
            addSurvey(self.portal, BASIC_SURVEY)

        survey_session = model.SurveySession(
            id=1,
            title="Dummy session",
            zodb_path="nl/ict/software-development",
            account=account,
        )
        model.Session.add(survey_session)

        traversed_session = self.portal.client.nl.ict[
            "software-development"
        ].restrictedTraverse("++session++1")

        # Tests
        with mock.patch(
            "euphorie.client.browser.webhelpers.WebHelpers.use_consultancy_phase",
            new_callable=mock.PropertyMock,
        ) as use_training_module:
            use_training_module.return_value = True
            with mock.patch(
                "euphorie.client.browser.webhelpers.WebHelpers.is_new_session",
                new_callable=mock.PropertyMock,
            ) as is_new_session:
                is_new_session.return_value = False
                with mock.patch(
                    "euphorie.client.browser.webhelpers.WebHelpers.get_phase",
                    return_value="preparation",
                ):
                    is_new_session.return_value = False
                    with api.env.adopt_user(user=account):
                        with self._get_view("webhelpers", traversed_session) as view:
                            nav_tree = view.survey_tree_data

                            self.assertEqual(nav_tree[0]["id"], "step-1")
                            self.assertEqual(nav_tree[1]["id"], "step-2")
                            self.assertEqual(nav_tree[2]["id"], "step-4")
                            self.assertEqual(nav_tree[3]["id"], "step-consultancy")
                            self.assertEqual(nav_tree[4]["id"], "step-5")
                            self.assertEqual(nav_tree[5]["id"], "status")

                            self.assertTrue("@@start" in nav_tree[0]["href"])
                            self.assertTrue("@@identification" in nav_tree[1]["href"])
                            self.assertTrue("@@actionplan" in nav_tree[2]["href"])
                            self.assertTrue("@@consultancy" in nav_tree[3]["href"])
                            self.assertTrue("@@report" in nav_tree[4]["href"])
                            self.assertTrue("@@status" in nav_tree[5]["href"])

                            self.assertEqual(nav_tree[0]["active"], True)
                            self.assertEqual(nav_tree[1]["active"], False)
                            self.assertEqual(nav_tree[2]["active"], False)
                            self.assertEqual(nav_tree[3]["active"], False)
                            self.assertEqual(nav_tree[4]["active"], False)
                            self.assertEqual(nav_tree[5]["active"], False)

                            self.assertEqual(nav_tree[0]["disabled"], False)
                            self.assertEqual(nav_tree[1]["disabled"], False)
                            self.assertEqual(nav_tree[2]["disabled"], False)
                            self.assertEqual(nav_tree[3]["disabled"], False)
                            self.assertEqual(nav_tree[4]["disabled"], False)
                            self.assertEqual(nav_tree[5]["disabled"], False)


class TestWebhelpersUnit(TestCase):
    def get_webhelpers(self, path):
        class DummyRequest:
            def __init__(self, path):
                self.PATH_INFO = path

        return WebHelpers(None, DummyRequest(path))

    def test_get_dashboard_tab(self):
        webhelpers = self.get_webhelpers(
            "/VirtualHostBase/https/oira.local:443/VirtualHostRoot/_vh_daimler"
            "/Plone/client/de/assessments"
        )
        self.assertEqual(webhelpers.get_dashboard_tab(), "assessments")

        webhelpers = self.get_webhelpers(
            "/VirtualHostBase/https/oira.local:443/VirtualHostRoot/_vh_daimler"
            "/Plone/client/de/assessments/"
        )
        self.assertEqual(webhelpers.get_dashboard_tab(), "assessments")

        webhelpers = self.get_webhelpers(
            "/VirtualHostBase/https/oira.local:443/VirtualHostRoot/_vh_daimler"
            "/Plone/client/de/@@assessments"
        )
        self.assertEqual(webhelpers.get_dashboard_tab(), "assessments")

        webhelpers = self.get_webhelpers(
            "/VirtualHostBase/https/oira.local:443/VirtualHostRoot/_vh_daimler"
            "/Plone/client/de/@@assessments/"
        )
        self.assertEqual(webhelpers.get_dashboard_tab(), "assessments")



from ftw.upgrade import UpgradeStep
from plone import api


class SetTheNewFieldEnableTestQuestions(UpgradeStep):
    """Set the new field enable_test_questions."""

    def __call__(self):
        """Upgrade step to set the new field enable_test_questions.

        If the web training is not enabled do not do anything.

        Otherwise check all the surveys that we have.
        If the survey web training is enabled and we have at least one
        training question in the survey, set the new field to True.
        """
        if not api.portal.get_registry_record(
            "euphorie.use_training_module", default=False
        ):
            return

        brains = api.content.find(portal_type="euphorie.survey")
        for brain in brains:
            survey = brain.getObject()
            if getattr(survey, "enable_web_training", True):
                if api.content.find(
                    context=survey, portal_type="euphorie.training_question"
                ):
                    survey.enable_test_questions = True



from euphorie.testing import EuphorieIntegrationTestCase
from plone import api
from plone.base.utils import get_installer
from time import time


class SetupTests(EuphorieIntegrationTestCase):
    def testDefaultContenetRemoved(self):
        self.assertTrue("Members" not in self.portal.objectIds())
        self.assertTrue("news" not in self.portal.objectIds())
        self.assertTrue("events" not in self.portal.objectIds())

    def testSectorContainerCreated(self):
        self.assertTrue("sectors" in self.portal.objectIds())
        self.assertEqual(self.portal.sectors.portal_type, "euphorie.sectorcontainer")

    def testCountriesCreated(self):
        self.assertTrue("nl" in self.portal.sectors)
        self.assertEqual(self.portal.sectors["nl"].country_type, "eu-member")

    def testClientCreated(self):
        self.assertTrue("client" in self.portal.objectIds())
        self.assertEqual(self.portal.client.portal_type, "euphorie.client")

    def testClientUserCreated(self):
        user = self.portal.acl_users.getUserById("client")
        self.assertTrue(user is not None)

    def testHideComponentProducts(self):
        qi = get_installer(self.portal)
        qi.is_product_installable("euphorie.client")
        self.assertFalse(qi.is_product_installable("euphorie.client"))
        self.assertFalse(qi.is_product_installable("euphorie.content"))

    def testNuPloneEnabled(self):
        st = self.portal.portal_skins
        self.assertEqual(st.getDefaultSkin(), "NuPlone")

    def test_get_resource_timestamp(self):
        self.logout()
        self.assertEqual(
            self.portal.restrictedTraverse("@@get-resources-timestamp")(), None
        )

        with api.env.adopt_user("admin"):
            self.portal.restrictedTraverse("@@refresh-resources-timestamp")()

        self.request.__annotations__.clear()
        self.assertEqual(
            int(self.portal.restrictedTraverse("@@get-resources-timestamp")()) // 10,
            int(time()) // 10,
        )



from ftw.upgrade import UpgradeStep
from plone import api


class ReImportSurveyGroup(UpgradeStep):
    """Re-import SurveyGroup type definition."""

    def __call__(self):
        # First, delete exisiting type registration for euphorie.surveygroup
        tt = api.portal.get_tool("portal_types")
        obj = tt.get("euphorie.surveygroup")
        if obj:
            api.content.delete(obj)
        self.install_upgrade_profile()



from Acquisition import aq_inner
from euphorie.content.utils import summarizeCountries
from Products.Five import BrowserView


class SectorContainerView(BrowserView):
    @property
    def countries(self):
        return summarizeCountries(aq_inner(self.context), self.request)



from ftw.upgrade import UpgradeStep


class AddedABrowserLayerForEuphorieContent(UpgradeStep):
    """Added a browser layer for euphorie.content."""

    def __call__(self):
        self.install_upgrade_profile()



from ftw.upgrade import UpgradeStep
from plone import api


class FixDefaultLanguage(UpgradeStep):
    """Fix default language."""

    def __call__(self):
        default_language = api.portal.get_registry_record("plone.default_language")
        available_languages = api.portal.get_registry_record(
            "plone.available_languages"
        )
        if default_language in available_languages:
            # No need to change the default language
            return

        # Try to pick a sane default language
        if "en" in available_languages:
            # Set the default language to English if it's available
            api.portal.set_registry_record("plone.default_language", "en")
        elif available_languages and len(available_languages) == 1:
            # If we have just one language available, set it as the default
            api.portal.set_registry_record(
                "plone.default_language", available_languages[0]
            )
        else:
            # If we have zero or more than one language available,
            # we require manual intervention
            raise ValueError(
                f"Cannot safely replace the default language {default_language}. "
                f"Please set up manually the registry records "
                f"`plone.default_language` and `plone.available_languages`."
            )



from euphorie.htmllaundry.cleaners import DocumentCleaner
from lxml import etree
from lxml import html
from lxml.html import defs

import re


INLINE_TAGS = defs.special_inline_tags | defs.phrase_tags | defs.font_style_tags
TAG = re.compile("<.*?>")
ANCHORS = etree.XPath(
    "descendant-or-self::a | descendant-or-self::x:a",
    namespaces={"x": html.XHTML_NAMESPACE},
)
ALL_WHITESPACE = re.compile(r"^\s*$", re.UNICODE)


def is_whitespace(txt):
    """Utility method to test if txt is all whitespace or None."""
    return txt is None or bool(ALL_WHITESPACE.match(txt))


def strip_markup(markup):
    """Strip all markup from a HTML fragment."""
    return TAG.sub("", markup)


StripMarkup = strip_markup  # BBB for htmllaundry <2.0


def remove_element(el):
    parent = el.getparent()
    if el.tail:
        previous = el.getprevious()
        if previous is not None:
            if previous.tail:
                previous.tail += el.tail
            else:
                previous.tail = el.tail
        else:
            if parent.text:
                parent.text += el.tail
            else:
                parent.text = el.tail

    parent.remove(el)


def remove_empty_tags(doc, extra_empty_tags=[]):
    """Removes all empty tags from a HTML document. Javascript editors
    and browsers have a nasty habit of leaving stray tags around after
    their contents have been removed. This function removes all such
    empty tags, leaving only valid empty tags.

    In addition consecutive <br/> tags are folded into a single tag.
    This forces whitespace styling to be done using CSS instead of via an
    editor, which almost always produces better and more consistent results.
    """
    empty_tags = {"br", "hr", "img", "input"}
    empty_tags.update(set(extra_empty_tags))
    legal_empty_tags = frozenset(empty_tags)

    if hasattr(doc, "getroot"):
        doc = doc.getroot()

    def clean(doc):
        victims = []
        for el in doc.iter():
            if el.tag == "br":
                preceding = el.getprevious()
                parent = el.getparent()

                if (
                    (preceding is None and not parent.text)
                    or (
                        preceding is not None
                        and preceding.tag == el.tag
                        and not preceding.tail
                    )
                    or (not el.tail and el.getnext() is None)
                ):
                    victims.append(el)
                    continue

            if el.tag in legal_empty_tags:
                continue

            # Empty <a> can be used as anchor.
            if (el.tag == "a") and (("name" in el.attrib) or ("id" in el.attrib)):
                continue

            if len(el) == 0 and is_whitespace(el.text):
                victims.append(el)
                continue

        if victims and victims[0] == doc:
            doc.clear()
            return 0
        else:
            for victim in victims:
                remove_element(victim)

        return len(victims)

    while clean(doc):
        pass

    return doc


def strip_outer_breaks(doc):
    """Remove any toplevel break elements."""
    victims = []

    for i in range(len(doc)):
        el = doc[i]
        if el.tag == "br":
            victims.append(el)

    for victim in victims:
        remove_element(victim)


MARKER = "LAUNDRY-INSERT"


def wrap_text(doc, element="p"):
    """Make sure there is no unwrapped text at the top level. Any bare text
    found is wrapped in a `<p>` element.
    """

    def par(text):
        el = etree.Element(element, {MARKER: ""})
        el.text = text
        return el

    if doc.text:
        doc.insert(0, par(doc.text))
        doc.text = None

    while True:
        for i, el in enumerate(doc):
            if html._nons(el.tag) in INLINE_TAGS and i and MARKER in doc[i - 1].attrib:
                doc[i - 1].append(el)
                break
            if not is_whitespace(el.tail):
                doc.insert(i + 1, par(el.tail))
                el.tail = None
                break
        else:
            break

    for el in doc:
        if MARKER in el.attrib:
            del el.attrib[MARKER]


def sanitize(input, cleaner=DocumentCleaner, wrap="p"):
    """Cleanup markup using a given cleanup configuration.
    Unwrapped text will be wrapped with wrap parameter.
    """
    if "body" not in cleaner.allow_tags:
        cleaner.allow_tags.append("body")

    input = "<html><body>%s</body></html>" % input
    document = html.document_fromstring(input)
    bodies = [e for e in document if html._nons(e.tag) == "body"]
    body = bodies[0]

    cleaned = cleaner.clean_html(body)
    remove_empty_tags(cleaned)
    strip_outer_breaks(cleaned)

    if wrap is not None:
        if wrap in html.defs.tags:
            wrap_text(cleaned, wrap)
        else:
            raise ValueError(
                "Invalid html tag provided for wrapping the sanitized text"
            )

    output = "".join(
        [etree.tostring(fragment, encoding=str) for fragment in cleaned.iterchildren()]
    )
    if wrap is None and cleaned.text:
        output = cleaned.text + output

    return output



from plone.app.upgrade.v52 import final


def rebuild_redirections(context):
    """For Euphorie does not have the registration tool (look for the
    disableRedirectTracking setuphandler), so we must skip this upgrade.

    This patch can be removed after the upgrade to Plone 5.2 is not
    needed anymore
    """
    pass


final.rebuild_redirections = rebuild_redirections



from euphorie.client import model
from euphorie.client.interfaces import IClientSkinLayer
from euphorie.client.tests.utils import addAccount
from euphorie.client.tests.utils import addSurvey
from euphorie.testing import EuphorieIntegrationTestCase
from plone import api
from z3c.saconfig import Session
from zope.interface import alsoProvides


SURVEY_1 = """
<sector xmlns="http://xml.simplon.biz/euphorie/survey/1.0">
  <title>sector</title>
  <survey>
    <title>survey</title>
    <language>nl</language>
    <module>
      <title>module1</title>
      <risk>
        <title>risk11</title>
        <problem_description>problem-desc-11</problem_description>
        <description>desc-11</description>
      </risk>
    </module>
    <module>
      <title>module2</title>
      <risk>
        <title>risk21</title>
        <problem_description>problem-desc-21</problem_description>
        <description>desc-21</description>
      </risk>
    </module>
  </survey>
</sector>
"""

SURVEY_2 = """
<sector xmlns="http://xml.simplon.biz/euphorie/survey/1.0">
  <title>sector</title>
  <survey>
    <title>survey</title>
    <language>nl</language>
    <module hide_from_training="True">
      <title>module1</title>
      <risk>
        <title>risk11</title>
        <problem_description>problem-desc-11</problem_description>
        <description>desc-11</description>
      </risk>
    </module>
    <module>
      <title>module2</title>
      <risk>
        <title>risk21</title>
        <problem_description>problem-desc-21</problem_description>
        <description>desc-21</description>
      </risk>
    </module>
  </survey>
</sector>
"""


class TestTrainingHideFromTraining(EuphorieIntegrationTestCase):
    """Tests the IHideFromTraining behavior.

    Also tests importing an xml structure with the hide_from_training
    attribute set.
    """

    def setUp(self):
        super().setUp()
        api.portal.set_registry_record("euphorie.use_training_module", True)

        # Add the hide_from_training behavior to the module type
        types = api.portal.get_tool("portal_types")
        fti_module = types.get("euphorie.module")
        fti_module.behaviors = tuple(
            list(fti_module.behaviors) + ["euphorie.hide_from_training"]
        )
        from plone.dexterity.schema import SCHEMA_CACHE

        SCHEMA_CACHE.invalidate("euphorie.module")

    def create_content(self, survey_xml):
        session = Session()
        self.account = addAccount(password="secret")
        with api.env.adopt_user("admin"):
            addSurvey(self.portal, survey_xml)
        self.survey = self.portal.client.nl.sector.survey
        alsoProvides(self.survey.REQUEST, IClientSkinLayer)

        survey_session = model.SurveySession(
            title="Dummy session",
            zodb_path="nl/sector/survey",
            account=self.account,
        )
        session.add(survey_session)

        module1 = survey_session.addChild(
            model.Module(title="module1", module_id="module1", zodb_path="1"),
        )
        module1.addChild(
            model.Risk(title="risk11", risk_id="risk11", zodb_path="1/2"),
        )
        module2 = survey_session.addChild(
            model.Module(title="module2", module_id="module2", zodb_path="3"),
        )
        module2.addChild(
            model.Risk(title="risk21", risk_id="risk21", zodb_path="3/4"),
        )
        session.flush()

    def test_training_module_visible(self):
        """Test the standard case - all modules and ristsk are visible in the
        training.
        """
        self.create_content(SURVEY_1)

        traversed_session = self.survey.restrictedTraverse("++session++1")
        with api.env.adopt_user(user=self.account):
            with self._get_view(
                "training", traversed_session, self.request.clone()
            ) as view:
                self.survey.enable_web_training = True
                data = list(view.slide_data.values())

                self.assertEqual(len(data), 4)
                self.assertEqual(data[0]["item"].title, "module1")
                self.assertEqual(data[1]["item"].title, "risk11")
                self.assertEqual(data[2]["item"].title, "module2")
                self.assertEqual(data[3]["item"].title, "risk21")

    def test_training_module_hidden(self):
        """Test module1 hidden from training."""
        self.create_content(SURVEY_2)

        traversed_session = self.survey.restrictedTraverse("++session++1")
        with api.env.adopt_user(user=self.account):
            with self._get_view(
                "training", traversed_session, self.request.clone()
            ) as view:
                self.survey.enable_web_training = True
                data = list(view.slide_data.values())

                self.assertEqual(len(data), 2)
                self.assertEqual(data[0]["item"].title, "module2")
                self.assertEqual(data[1]["item"].title, "risk21")



from AccessControl.Permission import addPermission
from zope.i18nmessageid import MessageFactory as mf


MessageFactory = mf("euphorie")
del mf


addPermission("Euphorie: Manage country", ("Manager",))
addPermission("Euphorie: Add new RIE Content", ("Manager",))
addPermission("Euphorie: Delete published content", ("Manager",))



from AccessControl.SecurityManagement import getSecurityManager
from AccessControl.SecurityManagement import newSecurityManager
from euphorie.content.tests.utils import addSurvey
from euphorie.content.tests.utils import createSector
from euphorie.testing import EuphorieIntegrationTestCase
from plone import api


class SurveyTests(EuphorieIntegrationTestCase):
    def createSurvey(self):
        sector = createSector(self.portal, login="sector")
        survey = addSurvey(sector)
        return survey

    def testSurveyWorkflow(self):
        self.loginAsPortalOwner()
        survey = self.createSurvey()
        pw = api.portal.get_tool("portal_workflow")
        chain = pw.getChainFor(survey)
        self.assertEqual(chain, ("survey",))

    def testNotGloballyAllowed(self):
        self.loginAsPortalOwner()
        types = [fti.id for fti in self.portal.allowedContentTypes()]
        self.assertTrue("euphorie.survey" not in types)

    def testAllowedContentTypes(self):
        self.loginAsPortalOwner()
        survey = self.createSurvey()
        types = [fti.id for fti in survey.allowedContentTypes()]
        self.assertEqual(set(types), {"euphorie.module", "euphorie.profilequestion"})

    def testCanDeleteItemsWhenNotPublished(self):
        self.loginAsPortalOwner()
        survey = self.createSurvey()
        sector = self.portal.acl_users.getUser("sector")
        newSecurityManager(None, sector)
        manager = getSecurityManager()
        self.assertTrue(manager.checkPermission("Delete objects", survey))



from euphorie.content import MessageFactory as _
from euphorie.content.vocabularies import RegistryValueVocabulary
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from zope import schema
from zope.interface import provider


@provider(IFormFieldProvider)
class IToolCategory(model.Schema):
    tool_category = schema.List(
        title=_("title_tool_category", default="Tool category"),
        description=_(
            "description_tool_category",
            default="Select one or more categories to which this OiRA tool "
            "is relevant. The user will then find this tool under the chosen"
            " categories.",
        ),
        required=False,
        value_type=schema.Choice(
            source=RegistryValueVocabulary(
                "euphorie.content.vocabularies.toolcategories",
            ),
        ),
    )



"""
Rich description
----------------

Several content types in euphorie use a rich text description. This
requires a little bit of special interaction for Plone, which is implemented
in this behaviour.

Enabling this behaviour for a content type does two things: it replaces
the standard description field with a rich text version, and registers
a catalog indexer which returns a plain text version of the description.
"""

from euphorie.content import MessageFactory as _
from euphorie.content.utils import StripMarkup
from plone.autoform import directives
from plone.indexer import indexer
from plone.supermodel import model
from plonetheme.nuplone.z3cform.widget import WysiwygFieldWidget
from zope import schema


try:
    from html import unescape
except ImportError:
    # PY2
    import HTMLParser

    unescape = HTMLParser.HTMLParser().unescape


class IRichDescription(model.Schema):
    """Simple behaviour for content types with a HTML description.

    Replaces the standard description field with a rich text version.
    """

    description = schema.Text(
        title=_("Summary"),
        description=_("A short summary of the content."),
        required=False,
    )
    directives.widget(description=WysiwygFieldWidget)
    directives.order_after(description="title")


@indexer(IRichDescription)
def Description(obj):
    """Indexer for rich text descriptions fields.

    Return a plain text version of the description for use in the Plone
    catalog.
    """
    d = getattr(obj, "description", "")
    if d is None:
        return None

    return unescape(StripMarkup(d))



"""
Survey Group
------------

A Survey Group is a container for several Survey versions.

https://admin.oiraproject.eu/sectors/eu/eu-private-security/private-security-eu
"""

from .. import MessageFactory as _
from Acquisition import aq_base
from Acquisition import aq_inner
from Acquisition import aq_parent
from plone.app.dexterity.behaviors.metadata import IBasic
from plone.autoform import directives
from plone.dexterity.content import Container
from plone.supermodel import model
from z3c.form.interfaces import IEditForm
from zope import schema
from zope.interface import implementer
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

import logging


log = logging.getLogger(__name__)


class RichTerm(SimpleTerm):
    def __init__(
        self, value, token=None, title=None, description=None, extra_help=None
    ):
        super().__init__(value, token, title)
        self.description = description
        self.extra_help = extra_help


class ISurveyGroup(model.Schema, IBasic):
    title = schema.TextLine(
        title=_("label_title", default="Title"),
        description=_(
            "help_surveygroup_title",
            default="The title of this OiRA Tool. This title is used in "
            "the OiRA Tool overview in the clients.",
        ),
        required=True,
    )
    directives.order_before(title="*")

    directives.omitted("description")

    obsolete = schema.Bool(
        title=_("label_survey_obsolete", default="Obsolete OiRA tool"),
        description=_(
            "help_survey_obsolete",
            default="This OiRA Tool is obsolete; it has been retired or "
            "replaced with another OiRA Tool.",
        ),
        default=False,
        required=False,
    )

    directives.omitted(IEditForm, "evaluation_algorithm")
    evaluation_algorithm = schema.Choice(
        title=_("label_survey_evaluation_algorithm", default="Evaluation algorithm"),
        vocabulary=SimpleVocabulary(
            [
                RichTerm(
                    "kinney",
                    title=_("algorithm_kinney", default="Standard three criteria"),
                    description=_(
                        "This is the recommended evaluation algorithm, "
                        "based on the Kinney method"
                    ),
                    extra_help=_(
                        "help_survey_evaluation_algorithm_standard",
                        default=(
                            "This method involves risk assessment "
                            "by Severity x Exposure x Probability, "
                            "whereas these have to be understood as follows; "
                            "Severity of injury linked to hazard - "
                            "Exposure (Frequency) to the hazard - "
                            "Probability of the hazard occuring when exposed"
                        ),
                    ),
                ),
                RichTerm(
                    "french",
                    title=_("algorithm_french", default="Simplified two criteria"),
                    description=_(
                        "This is a simpler evaluation algorithm "
                        "using only two criteria."
                    ),
                    extra_help=_(
                        "help_survey_evaluation_algorithm_simplified",
                        default=(
                            "This method involves risk assessment "
                            "by Severity x Exposure, whereas these have "
                            "to be understood as follows; Severity of injury linked "
                            "to hazard - Exposure (Frequency) to the hazard"
                        ),
                    ),
                ),
            ]
        ),
        default="kinney",
        required=True,
    )


@implementer(ISurveyGroup)
class SurveyGroup(Container):
    """A Survey Group is a container for several Survey versions."""

    published = None
    evaluation_algorithm = "kinney"
    obsolete = False

    def _canCopy(self, op=0):
        """Tell Zope2 that this object can not be copied."""
        return op

    @property
    def published_survey(self):
        """Return the published survey object or None if it does not exist."""
        published_id = getattr(aq_base(self), "published", "")
        if not published_id:
            return None

        try:
            published_obj = self[published_id]
        except KeyError:
            return None

        if published_obj.portal_type != "euphorie.survey":
            return None

        return published_obj

    def _delObject(self, obj_id, dp=1, suppress_events=False):  # type: ignore[override]
        """Do not allow to delete a published survey.

        If the survey is published or the last one, we raise an error.
        """
        try:
            obj = self[obj_id]
        except KeyError:
            obj = None

        if obj is not None and obj.portal_type == "euphorie.survey":
            if obj == self.published_survey:
                raise ValueError("You cannot delete a published survey.")
            surveys = [
                item
                for item in self.objectValues()
                if item.portal_type == "euphorie.survey"
            ]  # type: ignore[assignment]
            if len(surveys) == 1:
                raise ValueError("You cannot delete the last survey in a survey group.")

        return super(self.__class__, self)._delObject(obj_id, dp, suppress_events)


def handleSurveyPublish(survey, event):
    """Event handler (subscriber) for successfull workflow transitions for
    :py:obj:`ISurvey` objects. This handler performs necessary housekeeping
    tasks on the parent :py:class:`SurveyGroup`.

    If the workflow action is ``publish`` or ``update`` the ``published``
    attribute of the SurveyGroup is set to the id of the published
    survey instance.
    """
    if event.action not in ["publish", "update"]:
        return
    surveygroup = aq_parent(aq_inner(survey))
    surveygroup.published = survey.id


def handleSurveyRemoved(survey, event):
    """Event handler (subscriber) for deletion of
    :py:obj:`ISurvey` objects. This handler performs necessary houskeeping
    tasks on the parent :py:class:`SurveyGroup`.

    The 'published' attr on the surveygroup states the name of the currently
    published survey.

    If this survey gets deleted, we need to clear this attr.
    """
    parent = aq_parent(survey)
    if ISurveyGroup.providedBy(parent) and parent.published == survey.id:
        parent.published = None



from euphorie.client import model
from euphorie.client.browser.risk import IdentificationView
from euphorie.client.model import Risk
from euphorie.client.tests.utils import addAccount
from euphorie.client.tests.utils import addSurvey
from euphorie.content.risk import IFrenchEvaluation as IFr
from euphorie.content.risk import IKinneyEvaluation as IKi
from euphorie.content.survey import Survey
from euphorie.content.tests.utils import BASIC_SURVEY
from euphorie.testing import EuphorieIntegrationTestCase
from plone import api
from plone.base.utils import safe_text
from Products.CMFPlone.tests.dummy import Image
from unittest import mock
from zope.publisher.interfaces import NotFound

import unittest


class EvaluationViewTests(unittest.TestCase):
    def EvaluationView(self, **kw):
        """
        Note: the EvaluationView does not exist as a separate view any more.
        Its functionality was merged to the identification view.
        For these tests, we just keep the name EvaluationView but return
        IdentificationView, since it contains all the relevant code
        """
        return IdentificationView(Risk(**kw), None)

    def Risk(self, **kw):
        return Risk(**kw)

    def reply(self, freq, effect, prob=None):
        if prob is None:  # French
            return {
                "frequency": IFr["default_frequency"]
                .vocabulary.getTermByToken(freq)
                .value,
                "severity": IFr["default_severity"]
                .vocabulary.getTermByToken(effect)
                .value,
            }
        else:  # Kinney
            return {
                "frequency": IKi["default_frequency"]
                .vocabulary.getTermByToken(freq)
                .value,
                "effect": IKi["default_effect"].vocabulary.getTermByToken(effect).value,
                "probability": IKi["default_probability"]
                .vocabulary.getTermByToken(prob)
                .value,
            }

    def test_evaluation_algorithm_fallback(self):
        view = self.EvaluationView()
        risk = self.Risk()
        self.assertEqual(view.evaluation_algorithm(risk), "kinney")

    def test_evaluation_algorithm_survey_parent(self):
        view = self.EvaluationView()
        survey = Survey()
        survey.evaluation_algorithm = "dummy"
        risk = self.Risk().__of__(survey)
        self.assertEqual(view.evaluation_algorithm(risk), "dummy")

    def test_calculatePriority_kinney_nothing_set(self):
        view = self.EvaluationView()
        risk = self.Risk()
        risk.evaluation_method = "calculated"
        with mock.patch(
            "euphorie.client.browser.risk.evaluation_algorithm", return_value="kinney"
        ):
            self.assertEqual(view.calculatePriority(risk, {}), None)

    def test_calculatePriority_french_nothing_set(self):
        view = self.EvaluationView()
        risk = self.Risk()
        risk.evaluation_method = "calculated"
        with mock.patch(
            "euphorie.client.browser.risk.evaluation_algorithm", return_value="french"
        ):
            self.assertEqual(view.calculatePriority(risk, {}), None)

    def test_calculatePriority_french(self):
        view = self.EvaluationView()
        with mock.patch(
            "euphorie.client.browser.risk.evaluation_algorithm", return_value="french"
        ):
            risk = self.Risk()
            risk.evaluation_method = "calculated"
            # Risks with weak severity are always low priority
            for freq in ["rare", "not-often", "often", "regularly"]:
                self.assertEqual(
                    view.calculatePriority(risk, self.reply(freq, "weak")), "low"
                )
            # High priority items
            self.assertEqual(
                view.calculatePriority(risk, self.reply("often", "severe")), "high"
            )
            self.assertEqual(
                view.calculatePriority(risk, self.reply("often", "very-severe")), "high"
            )
            self.assertEqual(
                view.calculatePriority(risk, self.reply("regularly", "severe")), "high"
            )
            self.assertEqual(
                view.calculatePriority(risk, self.reply("regularly", "very-severe")),
                "high",
            )
            # Some medium priority items
            self.assertEqual(
                view.calculatePriority(risk, self.reply("rare", "very-severe")),
                "medium",
            )
            self.assertEqual(
                view.calculatePriority(risk, self.reply("not-often", "not-severe")),
                "medium",
            )

    def test_calculatePriority_kinney(self):
        view = self.EvaluationView()
        risk = self.Risk()
        risk.evaluation_method = "calculated"
        with mock.patch(
            "euphorie.client.browser.risk.evaluation_algorithm", return_value="kinney"
        ):
            # Risks with weak severity are always low priority
            for freq in ["almost-never", "regular", "constant"]:
                self.assertEqual(
                    view.calculatePriority(risk, self.reply(freq, "weak", "small")),
                    "low",
                )
            self.assertEqual(
                view.calculatePriority(
                    risk, self.reply("constant", "significant", "medium")
                ),
                "high",
            )
            self.assertEqual(
                view.calculatePriority(risk, self.reply("constant", "high", "medium")),
                "high",
            )
            self.assertEqual(
                view.calculatePriority(risk, self.reply("constant", "weak", "medium")),
                "medium",
            )


class TestRiskImageDownloadUpload(EuphorieIntegrationTestCase):
    def setUp(self):
        super().setUp()
        self.loginAsPortalOwner()
        self.account = addAccount(password="secret")
        addSurvey(self.portal, BASIC_SURVEY)
        survey_session = model.SurveySession(
            title="Dummy session",
            zodb_path="nl/ict/software-development",
            account=self.account,
        )
        self.risk = survey_session.addChild(
            model.Risk(title="risk 1", zodb_path="1")
        ).__of__(survey_session.traversed_session)
        model.Session.add(survey_session)

    def test_upload(self):
        with api.env.adopt_user(user=self.account):
            with self._get_view("image-upload", self.risk) as view:
                # Just calling the view with no request returns to the
                # @@identification page
                view()
                self.assertDictEqual(
                    view.request.response.headers,
                    {
                        "location": "http://nohost/plone/client/nl/ict/software-development/++session++1/1/@@identification"  # noqa: E501
                    },
                )

                # Uploading an image will save it to the risk
                view.request.form["image"] = Image()
                view()
                self.assertTrue(self.risk.image_data.startswith(b"GIF"))
                self.assertEqual(self.risk.image_filename, "dummy.gif")

                # We can also require to remove the image
                view.request.form.pop("image")
                view.request.form["image-remove"] = "1"
                view()
                self.assertIsNone(self.risk.image_data)
                self.assertFalse(self.risk.image_filename)

                # If we have a scale wipe it
                self.risk.image_data_scaled = b"foo"
                view.request.form["image"] = Image()
                view()
                self.assertEqual(self.risk.image_filename, "dummy.gif")
                self.assertIsNone(self.risk.image_data_scaled)

                # but do not wipe it if we are uploading the same image again
                self.risk.image_data_scaled = b"foo"
                view()
                self.assertEqual(self.risk.image_data_scaled, b"foo")

    def test_download(self):
        with api.env.adopt_user(user=self.account):
            with self._get_view("image-display", self.risk) as view:
                # If the risk has no image raise a not found
                with self.assertRaises(NotFound):
                    view()

                # Otherwise return the file
                self.risk.image_data = Image.data
                self.risk.image_filename = safe_text(Image.filename)
                self.assertTrue(view().startswith(b"GIF"))
                self.assertDictEqual(
                    view.request.response.headers,
                    {
                        "accept-ranges": "bytes",
                        "content-length": "168",
                        "content-type": "image/gif",
                        "link": '<http://nohost/plone/client/nl/ict/software-development/%2B%2Bsession%2B%2B1/1/@@download/None/dummy.gif>; rel="canonical"',  # noqa: E501
                    },
                )

                # Check that we can crop and scale the image on the fly
                view.fieldname = "image_training"
                self.assertTrue(view().startswith(b"\x89PNG"))

                headers = view.request.response.headers.copy()
                self.assertAlmostEqual(
                    int(headers.pop("content-length")), 6971, delta=10
                )
                self.assertDictEqual(
                    headers,
                    {
                        "accept-ranges": "bytes",
                        "content-type": "image/png",
                        "link": '<http://nohost/plone/client/nl/ict/software-development/%2B%2Bsession%2B%2B1/1/@@download/image_training/dummy.gif>; rel="canonical"',  # noqa: E501
                    },
                )



from ..utils import DragDropHelper
from .survey import SurveyBase
from euphorie.content import MessageFactory as _
from plone import api
from plone.dexterity.browser.add import DefaultAddForm
from plone.dexterity.browser.add import DefaultAddView
from plone.dexterity.browser.edit import DefaultEditForm
from plone.dexterity.interfaces import IDexterityFTI
from plone.memoize.instance import memoize
from Products.statusmessages.interfaces import IStatusMessage
from zope.component import getUtility


class ModuleView(SurveyBase, DragDropHelper):
    """View name: @@nuplone-view."""

    @property
    def portal_type(self):
        if self.context.aq_parent.portal_type == "euphorie.module":
            return _("Submodule")
        else:
            portal_type = self.context.portal_type
            fti = getUtility(IDexterityFTI, name=portal_type)
            return fti.Title()


class AddForm(DefaultAddForm):
    portal_type = "euphorie.module"

    @property
    def label(self):
        if self.context.portal_type == "euphorie.module":
            type_name = _("Submodule")
        else:
            portal_type = self.portal_type
            fti = getUtility(IDexterityFTI, name=portal_type)
            type_name = fti.Title()
        return _("Add %s" % type_name)


class AddView(DefaultAddView):
    form = AddForm


class EditForm(DefaultEditForm):
    """Override for the standard edit form so we can change the form title for
    submodules.

    View name: @@edit
    """

    @property
    def label(self):
        if self.context.aq_parent.portal_type == "euphorie.module":
            type_name = _("Submodule")
        else:
            portal_type = self.context.portal_type
            fti = getUtility(IDexterityFTI, name=portal_type)
            type_name = fti.Title()
        return _("Edit ${name}", mapping={"name": type_name})

    @property
    @memoize
    def portal_transforms(self):
        return api.portal.get_tool("portal_transforms")

    def get_safe_html(self, text):
        if not text:
            return ""
        data = self.portal_transforms.convertTo(
            "text/x-html-safe", text, mimetype="text/html"
        )
        return data.getData()

    def updateWidgets(self):
        super().updateWidgets()
        self.widgets["title"].addClass("span-7")
        for fname in ("description", "solution_direction"):
            value = self.widgets[fname].value or ""
            safe_value = self.get_safe_html(value)
            if value != safe_value:
                self.widgets[fname].value = safe_value

    def extractData(self, setErrors=True):
        data = super().extractData(setErrors)

        # If there is a validation error on the form, consume all status messages,
        # so that they don't appear in the form. We only want to show validation
        # messages directly on the respective field(s) in that case.
        if data[1]:
            status = IStatusMessage(self.request)
            status.show()
        return data



from Acquisition import aq_parent
from euphorie.content.browser.surveygroup import AddForm
from euphorie.testing import EuphorieFunctionalTestCase
from euphorie.testing import EuphorieIntegrationTestCase
from plone import api
from plone.base.utils import unrestricted_construct_instance
from plone.folder.interfaces import IExplicitOrdering
from Products.CMFCore.WorkflowCore import ActionSucceededEvent
from zope import component
from zope.event import notify


class SurveyGroupTests(EuphorieIntegrationTestCase):
    def setUp(self):
        super(self.__class__, self).setUp()
        self.client = self.portal.client
        self.loginAsPortalOwner()

    def _create(self, container, *args, **kwargs):
        newid = container.invokeFactory(*args, **kwargs)
        return getattr(container, newid)

    def createSurveyGroup(self):
        country = self.portal.sectors.nl
        sector = self._create(country, "euphorie.sector", "sector")
        surveygroup = self._create(sector, "euphorie.surveygroup", "group")
        return surveygroup

    def testNoWorkflow(self):
        surveygroup = self.createSurveyGroup()
        pw = api.portal.get_tool("portal_workflow")
        chain = pw.getChainFor(surveygroup)
        self.assertEqual(chain, ())

    def testNotGloballyAllowed(self):
        types = [fti.id for fti in self.portal.allowedContentTypes()]
        self.assertTrue("euphorie.survey" not in types)

    def testAllowedContentTypes(self):
        surveygroup = self.createSurveyGroup()
        types = [fti.id for fti in surveygroup.allowedContentTypes()]
        self.assertEqual(set(types), {"euphorie.survey"})

    def testCanNotBeCopied(self):
        surveygroup = self.createSurveyGroup()
        self.assertFalse(surveygroup.cb_isCopyable())


class HandleSurveyPublishTests(EuphorieIntegrationTestCase):
    def setUp(self):
        super(self.__class__, self).setUp()
        self.client = self.portal.client
        self.loginAsPortalOwner()

    def _create(self, container, *args, **kwargs):
        newid = container.invokeFactory(*args, **kwargs)
        return getattr(container, newid)

    def createSurveyGroup(self):
        country = self.portal.sectors.nl
        sector = self._create(country, "euphorie.sector", "sector")
        surveygroup = self._create(sector, "euphorie.surveygroup", "group")
        return surveygroup

    def testNothingPublished(self):
        surveygroup = self.createSurveyGroup()
        self.assertEqual(surveygroup.published, None)

    def testUnknownWorkflowAction(self):
        surveygroup = self.createSurveyGroup()
        survey = self._create(surveygroup, "euphorie.survey", "survey")
        notify(ActionSucceededEvent(survey, None, "bogus", None))
        self.assertEqual(surveygroup.published, None)

    def testPublishAction(self):
        surveygroup = self.createSurveyGroup()
        survey = self._create(surveygroup, "euphorie.survey", "survey")
        notify(ActionSucceededEvent(survey, None, "publish", None))
        self.assertEqual(surveygroup.published, "survey")

    def testUpdateAction(self):
        surveygroup = self.createSurveyGroup()
        survey = self._create(surveygroup, "euphorie.survey", "survey")
        notify(ActionSucceededEvent(survey, None, "update", None))
        self.assertEqual(surveygroup.published, "survey")

    def testUnpublishAction(self):
        surveygroup = self.createSurveyGroup()
        survey = self._create(surveygroup, "euphorie.survey", "survey")
        notify(ActionSucceededEvent(survey, None, "publish", None))
        self.assertEqual(surveygroup.published, "survey")
        request = survey.REQUEST
        unpublishview = component.getMultiAdapter(
            (surveygroup, request), name="unpublish"
        )
        unpublishview.unpublish()
        self.assertEqual(surveygroup.published, None)

    def testUndeletableSurveys(self):
        surveygroup = self.createSurveyGroup()
        one = self._create(surveygroup, "euphorie.survey", "one")

        # If you have just one survey, it cannot be deleted
        with self.assertRaises(ValueError):
            api.content.delete(one)

        # A survey group should only contain surveys, but theoretically there
        # could be other content.  You still cannot delete the only survey
        unrestricted_construct_instance("Image", surveygroup, "pretty")
        with self.assertRaises(ValueError):
            api.content.delete(one)

        two = self._create(surveygroup, "euphorie.survey", "two")
        # Unpublished surveys can be deleted
        api.content.delete(two)

        # Published surveys cannot be deleted
        three = self._create(surveygroup, "euphorie.survey", "three")
        notify(ActionSucceededEvent(three, None, "update", None))
        with self.assertRaises(ValueError):
            api.content.delete(three)


class HandleSurveyDeleteVerificationTests(EuphorieIntegrationTestCase):
    def setUp(self):
        super(self.__class__, self).setUp()
        self.client = self.portal.client
        self.loginAsPortalOwner()

    def _create(self, container, *args, **kwargs):
        newid = container.invokeFactory(*args, **kwargs)
        return getattr(container, newid)

    def createSurveyGroup(self):
        country = self.portal.sectors.nl
        sector = self._create(country, "euphorie.sector", "sector")
        surveygroup = self._create(sector, "euphorie.surveygroup", "group")
        return surveygroup

    def testDeleteOneOfManySurvey(self):
        """It should be possible to delete one of many surveys, when it's not
        published."""
        surveygroup = self.createSurveyGroup()
        self._create(surveygroup, "euphorie.survey", "dummy")
        survey = self._create(surveygroup, "euphorie.survey", "survey")
        self.assertEqual(surveygroup.published, None)
        deleteaction = component.getMultiAdapter(
            (survey, survey.REQUEST), name="delete"
        )
        self.assertEqual(deleteaction.verify(surveygroup, survey), True)

    def testDeleteOnlySurvey(self):
        """Validation should fail when trying to delete the only survey in a
        surveygroup."""
        surveygroup = self.createSurveyGroup()
        survey = self._create(surveygroup, "euphorie.survey", "survey")
        deleteaction = component.getMultiAdapter(
            (survey, survey.REQUEST), name="delete"
        )
        self.assertEqual(deleteaction.verify(surveygroup, survey), False)

    def testDeletePublishedSurvey(self):
        """Validation should fail when trying to delete a published survey."""
        surveygroup = self.createSurveyGroup()
        self._create(surveygroup, "euphorie.survey", "dummy")
        survey = self._create(surveygroup, "euphorie.survey", "survey")
        notify(ActionSucceededEvent(survey, None, "update", None))
        self.assertEqual(surveygroup.published, "survey")
        deleteaction = component.getMultiAdapter(
            (survey, survey.REQUEST), name="delete"
        )
        self.assertEqual(deleteaction.verify(surveygroup, survey), False)

    def testDeleteUnPublishedSurvey(self):
        """It should be possible to delete unpublished surveys."""
        surveygroup = self.createSurveyGroup()
        self._create(surveygroup, "euphorie.survey", "dummy")
        survey = self._create(surveygroup, "euphorie.survey", "survey")
        deleteaction = component.getMultiAdapter(
            (survey, survey.REQUEST), name="delete"
        )
        notify(ActionSucceededEvent(survey, None, "update", None))
        self.assertEqual(surveygroup.published, "survey")
        unpublishview = component.getMultiAdapter(
            (surveygroup, survey.REQUEST), name="unpublish"
        )
        unpublishview.unpublish()

        self.assertEqual(surveygroup.published, None)
        self.assertEqual(deleteaction.verify(surveygroup, survey), True)


class AddFormTests(EuphorieFunctionalTestCase):
    def setUp(self):
        super(self.__class__, self).setUp()
        self.client = self.portal.client
        self.loginAsPortalOwner()

    def _create(self, container, *args, **kwargs):
        newid = container.invokeFactory(*args, **kwargs)
        return getattr(container, newid)

    def createModule(self):
        country = self.portal.sectors.nl
        sector = self._create(country, "euphorie.sector", "sector")
        surveygroup = self._create(sector, "euphorie.surveygroup", "group")
        survey = self._create(surveygroup, "euphorie.survey", "survey")
        module = self._create(survey, "euphorie.module", "module")
        return module

    def testCopyResetsUIDs(self):
        module = self.createModule()
        self._create(module, "euphorie.risk", "one", title="one")
        request = module.REQUEST
        survey = aq_parent(module)
        container = self.portal.sectors.nl.sector
        target = self._create(container, "euphorie.surveygroup", "target")
        copy = AddForm(container, request).copyTemplate(survey, target)
        self.assertNotEqual(survey.UID(), copy.UID())
        self.assertNotEqual(survey.module.UID(), copy.module.UID())
        self.assertNotEqual(survey.module.one.UID(), copy.module.one.UID())

    def testCopyPreservesOrder(self):
        original_order = [
            "one",
            "two",
            "three",
            "four",
            "five",
            "six",
            "seven",
            "eight",
            "nine",
            "ten",
        ]
        module = self.createModule()
        for title in original_order:
            self._create(module, "euphorie.risk", title, title=title)
        self.assertEqual([r.title for r in module.values()], original_order)
        request = module.REQUEST
        survey = aq_parent(module)
        container = self.portal.sectors.nl.sector
        target = self._create(container, "euphorie.surveygroup", "target")
        copy = AddForm(container, request).copyTemplate(survey, target)
        self.assertEqual([r.title for r in copy["module"].values()], original_order)

    def testReorderThenCopyTemplateKeepsOrder(self):
        original_order = [
            "one",
            "two",
            "three",
            "four",
            "five",
            "six",
            "seven",
            "eight",
            "nine",
            "ten",
        ]
        sorted_order = list(sorted(original_order))
        module = self.createModule()
        for title in original_order:
            self._create(module, "euphorie.risk", title, title=title)
        self.assertEqual([r.title for r in module.values()], original_order)
        ordering = IExplicitOrdering(module)
        ordering.orderObjects("title")
        self.assertEqual([r.title for r in module.values()], sorted_order)
        request = module.REQUEST
        survey = aq_parent(module)
        container = self.portal.sectors.nl.sector
        target = self._create(container, "euphorie.surveygroup", "target")
        copy = AddForm(container, request).copyTemplate(survey, target)
        self.assertEqual([r.title for r in copy["module"].values()], sorted_order)

    def testCopyClearsPublishFlag(self):
        survey = aq_parent(self.createModule())
        survey.published = True
        request = survey.REQUEST
        container = self.portal.sectors.nl.sector
        target = self._create(container, "euphorie.surveygroup", "target")
        copy = AddForm(container, request).copyTemplate(survey, target)
        self.assertFalse(getattr(copy, "published", False))

    def testCopyResetsWorkflow(self):
        survey = aq_parent(self.createModule())
        survey.published = True
        request = survey.REQUEST
        container = self.portal.sectors.nl.sector
        target = self._create(container, "euphorie.surveygroup", "target")
        copy = AddForm(container, request).copyTemplate(survey, target)
        self.assertEqual(
            self.portal.portal_workflow.getInfoFor(copy, "review_state"), "draft"
        )

    def testCopyEvaluationAlgorithmFromGroup(self):
        survey = aq_parent(self.createModule())
        survey.aq_parent.evaluation_algorithm = "french"
        request = survey.REQUEST
        container = self.portal.sectors.nl.sector
        target = self._create(
            container, "euphorie.surveygroup", "target", evaluation_algorithm="kinney"
        )
        AddForm(container, request).copyTemplate(survey, target)
        self.assertEqual(target.evaluation_algorithm, "french")



"""
Country Manager
---------------

A `membrane` user account inside a country that has the permission to add
sectors (which are also membrane users) to that country.

.. _membrane: http://pypi.python.org/pypi/Products.membrane
"""

from Acquisition import aq_inner
from borg.localrole.interfaces import ILocalRoleProvider
from euphorie.content.country import ICountry
from euphorie.content.user import IUser
from euphorie.content.user import UserProvider
from plone.dexterity.content import Item
from plone.indexer import indexer
from plone.supermodel import model
from plone.uuid.interfaces import IAttributeUUID
from Products.CMFCore.utils import getToolByName
from zope.component import adapter
from zope.interface import implementer


class ICountryManager(model.Schema, IUser):
    """A country manager is responsible for managing sectors in their
    country."""


@implementer(ICountryManager, IAttributeUUID)
class CountryManager(Item):
    """A country manager."""

    portal_type = "euphorie.countrymanager"

    locked = False

    def _canCopy(self, op=0):
        """Tell Zope2 that this object can not be copied."""
        return False


@indexer(ICountryManager)
def SearchableTextIndexer(obj):
    """Index the title, contact_name and contact_email."""
    return " ".join([obj.title, obj.contact_name, obj.contact_email])


@adapter(ICountry)
@implementer(ILocalRoleProvider)
class CountryManagerLocalRoleProvider:
    """`borg.localrole` provider for :py:class:`ICountryManager` instances.

    This local role provider gives country managers the Administrator
    role within their country.
    """

    def __init__(self, context):
        self.context = context

    def getRoles(self, principal_id):
        mt = getToolByName(self.context, "membrane_tool")
        user = mt.getUserObject(user_id=principal_id, brain=True)
        if user is None:
            return ()

        user = user._unrestrictedGetObject()
        if not ICountryManager.providedBy(user):
            return ()

        if user.getPhysicalPath()[:-1] == aq_inner(self.context).getPhysicalPath():
            return ("CountryManager", "Editor", "Contributor", "Reader", "Reviewer")
        else:
            return ()

    def getAllRoles(self):
        managers = [
            UserProvider(manager)
            for manager in self.context.values()
            if ICountryManager.providedBy(manager)
        ]
        return [
            (
                manager.getUserId(),
                ("CountryyManager", "Editor", "Contributor", "Reader", "Reviewer"),
            )
            for manager in managers
        ]



from AccessControl.PermissionRole import _what_not_even_god_should_do
from datetime import timedelta
from euphorie.client import config
from euphorie.client import model
from euphorie.client.tests.database import DatabaseTests
from euphorie.testing import EuphorieIntegrationTestCase
from plone import api
from plone.app.event.base import localized_now
from sqlalchemy.exc import StatementError
from unittest import mock
from z3c.saconfig import Session


def createSurvey():
    session = Session()
    account = model.Account(loginname="jane", password="secret")
    session.add(account)
    survey = model.SurveySession(title="Session", zodb_path="survey", account=account)
    session.add(survey)
    session.flush()
    return (session, survey)


class SurveySessionTests(EuphorieIntegrationTestCase):
    def test_iface(self):
        """SurveySessions are marked by the ISurveySession interface."""
        survey = createSurvey()[-1]
        self.assertTrue(model.ISurveySession.providedBy(survey))

    def test_is_archived(self):
        """Verify that a session is archived when the archived attribute is set
        and it is in the past."""
        session = model.SurveySession()
        self.assertIsNone(session.archived)
        self.assertFalse(session.is_archived)
        session.archived = localized_now()
        self.assertTrue(session.is_archived)
        session.archived += timedelta(days=1)
        self.assertFalse(session.is_archived)

    def test_get_context_filter(self):
        with api.env.adopt_user("admin"):
            eu = api.content.create(
                type="euphorie.clientcountry", id="eu", container=self.portal.client
            )
            eusector = api.content.create(
                type="euphorie.clientsector", id="eusector", container=eu
            )
            api.content.create(
                type="euphorie.survey", id="eusurvey", container=eusector
            )
            nl = api.content.create(
                type="euphorie.clientcountry", id="nl", container=self.portal.client
            )
            nlsector = api.content.create(
                type="euphorie.clientsector", id="nlsector", container=nl
            )
            api.content.create(
                type="euphorie.survey", id="nlsurvey", container=nlsector
            )

            context_filter = model.SurveySession.get_context_filter(self.portal)
            self.assertSetEqual(
                set(context_filter.right.value),
                {"eu/eusector/eusurvey", "nl/nlsector/nlsurvey"},
            )

            context_filter = model.SurveySession.get_context_filter(self.portal.client)
            self.assertSetEqual(
                set(context_filter.right.value),
                {"eu/eusector/eusurvey", "nl/nlsector/nlsurvey"},
            )

            context_filter = model.SurveySession.get_context_filter(
                self.portal.client.eu
            )
            self.assertSetEqual(
                set(context_filter.right.value),
                {"eu/eusector/eusurvey"},
            )

            context_filter = model.SurveySession.get_context_filter(self.portal.sectors)
            self.assertFalse(context_filter)

    def testNoChildren(self):
        (ses, survey) = createSurvey()
        root = survey.addChild(model.Module(title="Root", module_id="1", zodb_path="1"))
        ses.add(root)
        ses.flush()
        self.assertEqual(root.children().count(), 0)

    def testAddChild(self):
        (ses, survey) = createSurvey()
        root = survey.addChild(model.Module(title="Root", module_id="1", zodb_path="1"))
        ses.add(root)
        root.addChild(model.Module(title="Module", module_id="1", zodb_path="1/1"))
        ses.flush()
        self.assertEqual(root.children().count(), 1)

    def testChildOrder(self):
        (ses, survey) = createSurvey()
        root = survey.addChild(model.Module(title="Root", module_id="1", zodb_path="1"))
        ses.add(root)
        ses.flush()
        root.addChild(model.Module(title="Profile 5", module_id="5", zodb_path="1/5"))
        root.addChild(model.Module(title="Profile 1", module_id="1", zodb_path="1/1"))
        root.addChild(model.Module(title="Profile 3", module_id="3", zodb_path="1/3"))
        ses.flush()
        self.assertEqual([c.module_id for c in list(root.children())], ["5", "1", "3"])

    def testReset_NoChildren(self):
        (ses, survey) = createSurvey()
        survey.reset()
        children = ses.query(model.SurveyTreeItem.id).filter(
            model.SurveyTreeItem.session == survey
        )
        self.assertEqual(children.count(), 0)

    def testReset_SingleChild(self):
        (ses, survey) = createSurvey()
        root = survey.addChild(model.Module(title="Root", module_id="1", zodb_path="1"))
        ses.add(root)
        children = ses.query(model.SurveyTreeItem.id).filter(
            model.SurveyTreeItem.session == survey
        )
        self.assertEqual(children.count(), 1)
        survey.reset()
        self.assertEqual(children.count(), 0)

    def testHasTree_NoChildren(self):
        (ses, survey) = createSurvey()
        self.assertEqual(survey.hasTree(), False)

    def testHasTree_SingleChild(self):
        (ses, survey) = createSurvey()
        root = survey.addChild(model.Module(title="Root", module_id="1", zodb_path="1"))
        ses.add(root)
        self.assertEqual(survey.hasTree(), True)

    def test_absolute_url(self):
        survey = createSurvey()[1]
        with self.assertRaises(ValueError):
            survey.absolute_url()

        # We now set the zodb_path to something
        # traversable in the context of the client folder
        survey.zodb_path = ""

        self.assertEqual(
            survey.absolute_url(), "http://nohost/plone/client/++session++1"
        )


class RiskPresentFilterTests(EuphorieIntegrationTestCase):
    def createData(self):
        (self.session, self.survey) = createSurvey()
        self.mod1 = model.Module(
            title="Module 1", module_id="1", zodb_path="1", skip_children=False
        )
        self.survey.addChild(self.mod1)
        self.q1 = model.Risk(
            title="Risk 1",
            risk_id="1",
            zodb_path="1/1",
            type="risk",
            identification="no",
        )
        self.mod1.addChild(self.q1)

    def query(self):
        return self.session.query(model.SurveyTreeItem).filter(
            model.RISK_PRESENT_FILTER
        )

    def testValidRisk(self):
        self.createData()
        self.assertEqual(self.query().count(), 1)

    def testIncludeTop5(self):
        self.createData()
        self.q1.risk_type = "top5"
        self.assertEqual(self.query().count(), 1)

    def testNoPresentRisk(self):
        self.createData()
        self.q1.identification = "n/a"
        self.assertEqual(self.query().count(), 0)


class RiskPresentNoTop5FilterTests(EuphorieIntegrationTestCase):
    def createData(self):
        (self.session, self.survey) = createSurvey()
        self.mod1 = model.Module(
            title="Module 1", module_id="1", zodb_path="1", skip_children=False
        )
        self.survey.addChild(self.mod1)
        self.q1 = model.Risk(
            title="Risk 1",
            risk_id="1",
            zodb_path="1/1",
            type="risk",
            identification="no",
        )
        self.mod1.addChild(self.q1)

    def query(self):
        return self.session.query(model.SurveyTreeItem).filter(
            model.RISK_PRESENT_NO_TOP5_NO_POLICY_DO_EVALUTE_FILTER
        )

    def testValidRisk(self):
        self.createData()
        self.assertEqual(self.query().count(), 1)

    def testSkipTop5(self):
        self.createData()
        self.q1.risk_type = "top5"
        self.assertEqual(self.query().count(), 0)

    def testSkipPolicy(self):
        self.createData()
        self.q1.risk_type = "policy"
        self.assertEqual(self.query().count(), 0)

    def testNoPresentRisk(self):
        self.createData()
        self.q1.identification = "n/a"
        self.assertEqual(self.query().count(), 0)


class ModuleWithRiskFilterTests(EuphorieIntegrationTestCase):
    def createData(self):
        (self.session, self.survey) = createSurvey()
        self.mod1 = model.Module(
            title="Module 1", module_id="1", zodb_path="1", skip_children=False
        )
        self.survey.addChild(self.mod1)
        self.q1 = model.Risk(
            title="Risk 1",
            risk_id="1",
            zodb_path="1/1",
            type="risk",
            identification="no",
        )
        self.mod1.addChild(self.q1)

    def query(self):
        return self.session.query(model.SurveyTreeItem).filter(
            model.MODULE_WITH_RISK_FILTER
        )

    def testValidModule(self):
        self.createData()
        self.assertEqual(self.query().count(), 1)

    def testSkipChildren(self):
        self.createData()
        self.mod1.skip_children = True
        self.assertEqual(self.query().count(), 0)

    def testNoPresentRisk(self):
        self.createData()
        self.q1.identification = "n/a"
        self.assertEqual(self.query().count(), 0)

    def testNoChildren(self):
        (self.session, self.survey) = createSurvey()
        self.session.add(self.survey)
        self.mod1 = model.Module(
            title="Module 1", module_id="1", zodb_path="1", skip_children=False
        )
        self.survey.addChild(self.mod1)
        self.assertEqual(self.query().count(), 0)


class AccountTests(DatabaseTests):
    def testGlobalRoles(self):
        user = model.Account()
        self.assertEqual(user.getRoles(), ("EuphorieUser",))

    def testRolesInContext(self):
        user = model.Account()
        self.assertEqual(user.getRolesInContext(None), ("EuphorieUser",))

    def testNoViewPermission(self):
        user = model.Account()
        self.assertEqual(user.has_permission("View", None), False)

    def testHasSurveyViewPermission(self):
        user = model.Account()
        self.assertEqual(user.has_permission("Euphorie: View a Survey", None), True)

    def testAccountType(self):
        (self.session, self.survey) = createSurvey()
        account = self.survey.account
        self.assertEqual(account.account_type, config.FULL_ACCOUNT)
        account.account_type = config.GUEST_ACCOUNT
        self.assertEqual(account.account_type, config.GUEST_ACCOUNT)
        self.session.flush()  # check that exception is not raised
        account.account_type = config.CONVERTED_ACCOUNT
        self.session.flush()  # check that exception is not raised
        self.assertEqual(account.account_type, config.CONVERTED_ACCOUNT)
        account.account_type = "invalid"
        self.assertRaises(StatementError, self.session.flush)

    def testGroups(self):
        session = Session()
        group1 = model.Group(group_id="1")
        session.add(group1)
        session.flush()
        self.assertEqual(group1.group_id, "1")
        # Verify that a group might have one parent but many children
        group2 = model.Group(group_id="2")
        session.add(group2)
        group3 = model.Group(group_id="3")
        session.add(group3)
        group3.parent = group2.parent = group1
        session.flush()
        self.assertEqual(group2.group_id, "2")
        self.assertEqual(group2.parent, group1)
        self.assertEqual(group1.parent, None)
        self.assertListEqual(group1.children, [group2, group3])
        self.assertListEqual(group2.children, [])

    def testAccountGroupsRelationship(self):
        session = Session()
        group1 = model.Group(group_id="1")
        session.add(group1)
        session.flush()
        account1 = model.Account(loginname="account1")
        session.add(account1)
        account2 = model.Account(loginname="account2")
        session.add(account2)
        session.flush()
        self.assertEqual(account1.group, None)
        account1.group = group1
        account2.group = group1
        self.assertListEqual(group1.accounts, [account1, account2])

    def testAccountGroupsHierarchy(self):
        session = Session()
        group1 = model.Group(group_id="1")
        session.add(group1)
        group2 = model.Group(group_id="2")
        group2.parent = group1
        session.add(group2)
        group3 = model.Group(group_id="3")
        session.add(group3)
        group3.parent = group1
        group4 = model.Group(group_id="4")
        session.add(group4)
        group4.parent = group2
        session.flush()
        account1 = model.Account(loginname="account1")
        session.add(account1)
        account1.group = group1
        account2 = model.Account(loginname="account2")
        session.add(account2)
        account2.group = group2
        account3 = model.Account(loginname="account3")
        session.add(account3)
        account3.group = group3
        account4 = model.Account(loginname="account4")
        session.add(account4)
        session.flush()
        self.assertListEqual(group1.descendants, [group2, group3, group4])
        self.assertListEqual(group2.descendants, [group4])
        self.assertListEqual(group3.descendants, [])
        self.assertListEqual(account1.groups, [group1, group2, group3, group4])
        self.assertListEqual(account2.groups, [group2, group4])
        self.assertListEqual(account3.groups, [group3])
        self.assertListEqual(account4.groups, [])
        self.assertListEqual(group1.parents, [])
        self.assertListEqual(group2.parents, [group1])
        self.assertListEqual(group3.parents, [group1])
        self.assertListEqual(group4.parents, [group2, group1])

    def testSessions(self):
        session = Session()
        group1 = model.Group(group_id="1")
        session.add(group1)
        group2 = model.Group(group_id="2")
        session.add(group2)
        account1 = model.Account(loginname="account1")
        session.add(account1)
        account1.group = group1
        group2.parent = group1
        from functools import partial

        add_survey = partial(model.SurveySession, account=account1)
        survey1 = add_survey(zodb_path="1")
        session.add(survey1)
        survey2 = add_survey(zodb_path="2", group=group1)
        session.add(survey2)
        survey3 = add_survey(zodb_path="3", group=group2)
        session.add(survey3)
        session.flush()
        self.assertListEqual(account1.sessions, [survey1, survey2, survey3])
        self.assertListEqual(group1.sessions, [survey2])
        self.assertListEqual(group2.sessions, [survey3])

    def testSessionAcquisition(self):
        """Users belonging to a group should be able to see all the sessions
        belonging to the group and the group children."""
        session = Session()
        group1 = model.Group(group_id="1")
        session.add(group1)
        group2 = model.Group(group_id="2")
        session.add(group2)
        account1 = model.Account(loginname="account1")
        session.add(account1)
        account1.group = group1
        group2.parent = group1
        account2 = model.Account(loginname="account2")
        session.add(account2)
        account2.group = group2
        survey1 = model.SurveySession(
            account=account1,
            group=group1,
            zodb_path="1",
        )
        session.add(survey1)
        survey2 = model.SurveySession(
            account=account2,
            group=group2,
            zodb_path="2",
        )
        session.add(survey2)
        session.flush()
        self.assertListEqual(account1.sessions, [survey1])
        self.assertListEqual(account2.sessions, [survey2])
        self.assertListEqual(account1.acquired_sessions, [survey1, survey2])
        self.assertListEqual(account2.acquired_sessions, [survey2])


class AccountIntegrationTests(EuphorieIntegrationTestCase):
    def setUp(self):
        super().setUp()
        self.allowed = model.Account().allowed
        self.client = self.portal.client

    def test_not_licet_iovi_not_licet_bovi(self):
        self.assertFalse(self.allowed(self.client, _what_not_even_god_should_do))

    def test_has_anonymous_role(self):
        self.assertTrue(self.allowed(self.client, ["Anonymous"]))

    def test_has_authenticated_role(self):
        self.assertTrue(self.allowed(self.client, ["Authenticated"]))

    def test_has_euphorie_user_role(self):
        self.assertTrue(self.allowed(self.client, ["EuphorieUser"]))

    def test_has_reader_role(self):
        self.assertTrue(self.allowed(self.client, ["EuphorieUser"]))

    def test_no_other_role(self):
        self.assertFalse(self.allowed(self.client, ["Manager"]))

    def test_has_no_reader_role_on_portal(self):
        self.assertFalse(self.allowed(self.portal, ["Reader"]))


class SurveySessionDBTests(DatabaseTests):
    def test_get_account_filter(self):
        session = model.SurveySession()
        account = model.Account(id=1)
        # Note assertFalse will not do what you want on Binary expression
        self.assertEqual(str(session.get_account_filter()), "False")
        self.assertEqual(str(session.get_account_filter(False)), "False")
        self.assertEqual(str(session.get_account_filter(None)), "False")
        self.assertEqual(str(session.get_account_filter("")), "False")
        self.assertEqual(str(session.get_account_filter([])), "False")
        self.assertEqual(str(session.get_account_filter([""])), "False")
        self.assertEqual(
            str(session.get_account_filter("1")), "session.account_id = :account_id_1"
        )
        self.assertEqual(
            str(session.get_account_filter(1)), "session.account_id = :account_id_1"
        )
        session.get_account_filter(account)
        self.assertEqual(
            str(session.get_account_filter(account)),
            "session.account_id = :account_id_1",
        )
        self.assertEqual(
            str(session.get_account_filter(["1"])), "session.account_id = :account_id_1"
        )
        self.assertEqual(
            str(session.get_account_filter([1])), "session.account_id = :account_id_1"
        )
        self.assertEqual(
            str(session.get_account_filter([account])),
            "session.account_id = :account_id_1",
        )
        self.assertEqual(
            str(session.get_account_filter([account, "2"])),
            "session.account_id IN (__[POSTCOMPILE_account_id_1])",
        )
        with mock.patch("euphorie.client.model.get_current_account", return_value=None):
            self.assertEqual(str(session.get_account_filter(True)), "False")
        with mock.patch(
            "euphorie.client.model.get_current_account", return_value=account
        ):
            self.assertEqual(
                str(session.get_account_filter(True)),
                "session.account_id = :account_id_1",
            )
        with self.assertRaises(TypeError):
            session.get_account_filter(session)

    def test_get_group_filter(self):
        session = model.SurveySession()
        group = model.Group(group_id="foo")
        account = model.Account(id=1)
        # Note assertFalse will not do what you want on Binary expression
        self.assertEqual(str(session.get_group_filter()), "False")
        self.assertEqual(str(session.get_group_filter(False)), "False")
        self.assertEqual(str(session.get_group_filter(None)), "False")
        self.assertEqual(str(session.get_group_filter("")), "False")
        self.assertEqual(str(session.get_group_filter([])), "False")
        self.assertEqual(str(session.get_group_filter([""])), "False")
        self.assertEqual(
            str(session.get_group_filter("1")), "session.group_id = :group_id_1"
        )
        self.assertEqual(
            str(session.get_group_filter(1)), "session.group_id = :group_id_1"
        )
        self.assertEqual(
            str(session.get_group_filter(group)),
            "session.group_id = :group_id_1",
        )
        self.assertEqual(
            str(session.get_group_filter(["1"])), "session.group_id = :group_id_1"
        )
        self.assertEqual(
            str(session.get_group_filter([1])), "session.group_id = :group_id_1"
        )
        self.assertEqual(
            str(session.get_group_filter([group])),
            "session.group_id = :group_id_1",
        )
        self.assertEqual(
            str(session.get_group_filter([group, "2"])),
            "session.group_id IN (__[POSTCOMPILE_group_id_1])",
        )
        with mock.patch("euphorie.client.model.get_current_account", return_value=None):
            self.assertEqual(str(session.get_group_filter(True)), "False")

        # The account still does not have a group, so we should have False here
        with mock.patch(
            "euphorie.client.model.get_current_account", return_value=account
        ):
            self.assertEqual(str(session.get_group_filter(True)), "False")

        account.group_id = "foo"
        with mock.patch(
            "euphorie.client.model.get_current_account", return_value=account
        ):
            self.assertEqual(
                str(session.get_group_filter(True)),
                "session.group_id = :group_id_1",
            )

        with self.assertRaises(TypeError):
            session.get_group_filter(session)



from datetime import datetime
from euphorie.client import model
from euphorie.client.tests.utils import addAccount
from euphorie.client.tests.utils import addSurvey
from euphorie.content.tests.utils import BASIC_SURVEY
from euphorie.testing import EuphorieIntegrationTestCase
from lxml import html
from plone import api
from Products.Five.browser.metaconfigure import ViewNotCallableError
from time import sleep
from zExceptions.unauthorized import Unauthorized
from zope.event import notify
from zope.lifecycleevent import ObjectModifiedEvent


class TestSurveyViews(EuphorieIntegrationTestCase):
    def _make_traversed_survey_session(self):
        with api.env.adopt_user("admin"):
            survey = addSurvey(self.portal, BASIC_SURVEY)
        account = addAccount(password="secret")
        survey_session = model.SurveySession(
            id=123,
            title="Dummy session",
            created=datetime(2012, 4, 22, 23, 5, 12),
            modified=datetime(2012, 4, 23, 11, 50, 30),
            zodb_path="nl/ict/software-development",
            account=account,
            company=model.Company(country="nl", employees="1-9", referer="other"),
        )
        model.Session.add(survey_session)
        survey = self.portal.client.nl.ict["software-development"]
        session_id = "++session++%d" % survey_session.id
        traversed_survey_session = survey.restrictedTraverse(session_id)
        return traversed_survey_session

    def test_survey_locking_views_when_feature_disabled(self):
        """The view is not callable but it has allowed attributes.
        They will anyway raise Unauthorized if the feature is not toggled on
        """
        traversed_survey_session = self._make_traversed_survey_session()
        survey_session = traversed_survey_session.session
        with api.env.adopt_user(user=survey_session.account):
            with self._get_view(
                "locking_view", traversed_survey_session, survey_session
            ) as view:
                # The view is not callable but
                # has traversable allowed attributes
                with self.assertRaises(ViewNotCallableError):
                    view()
                with self.assertRaises(Unauthorized):
                    view.set_lock()
                with self.assertRaises(Unauthorized):
                    view.refresh_lock()
                with self.assertRaises(Unauthorized):
                    view.unset_lock()

    def test_survey_locking_views_when_feature_enabled(self):
        """We have some views to display and set the locked state
        for a survey session."""
        traversed_survey_session = self._make_traversed_survey_session()
        survey_session = traversed_survey_session.session

        # We need to turn on the locking feature to access the view methods
        api.portal.set_registry_record("euphorie.use_locking_feature", True)

        with api.env.adopt_user(user=survey_session.account):
            with self._get_view(
                "locking_view", traversed_survey_session, survey_session
            ) as view:
                # The view is not callable but
                # has traversable allowed attributes
                self.assertRaises(ViewNotCallableError, view)
                # We have some column/properties that are going to change
                # when locking/unlocking the session
                self.assertEqual(survey_session.last_modifier, None)
                self.assertFalse(survey_session.is_locked)
                # Now we create a new locking event
                # If no referer is set,
                # the methods will redirect to the context url
                self.assertEqual(
                    view.set_lock(), traversed_survey_session.absolute_url()
                )
                self.assertEqual(
                    survey_session.last_locking_event.account, survey_session.account
                )
                self.assertTrue(survey_session.is_locked)
                old_lock_time = survey_session.last_locking_event.time

                view.request.__annotations__.clear()
                # Changing the HTTP_REFERER will redirect there
                # and calling refresh_lock will update the locking date
                view.request.set("HTTP_REFERER", "foo")
                # We need to wait at least one second because the datetime
                # is stored with that accuracy
                sleep(1)
                self.assertEqual(view.refresh_lock(), "foo")
                self.assertLess(old_lock_time, survey_session.last_locking_event.time)

                sleep(1)
                view.request.__annotations__.clear()
                # Calling unset_date will restore the unlocked state info
                self.assertEqual(view.unset_lock(), "foo")
                self.assertFalse(survey_session.is_locked)

            # We also have a menu view
            with self._get_view(
                "locking_menu", traversed_survey_session, survey_session
            ) as view:
                soup = html.fromstring(view())
                self.assertListEqual(
                    ["locking_view/set_lock#content"],
                    [
                        el.attrib["action"].rpartition("@@")[-1]
                        for el in soup.cssselect("form")
                    ],
                )

            # We trigger the session to be private
            with self._get_view(
                "locking_view", traversed_survey_session, survey_session
            ) as view:
                sleep(1)
                view.set_lock()

            # Now the locking menu will have options to uinlock and refresh lock
            with self._get_view(
                "locking_menu", traversed_survey_session, survey_session
            ) as view:
                soup = html.fromstring(view())
                self.assertListEqual(
                    [
                        "locking_view/unset_lock#content",
                        "locking_view/refresh_lock#content",
                    ],
                    [
                        el.attrib["action"].rpartition("@@")[-1]
                        for el in soup.cssselect("form")
                    ],
                )

    def test_modify_updates_last_modifier(self):
        account = addAccount(password="secret")
        survey_session = model.SurveySession(
            title="Dummy session", account=account, zodb_path=""
        )
        self.assertEqual(survey_session.modified, None)
        self.assertEqual(survey_session.last_modifier, None)
        with api.env.adopt_user(user=account):
            notify(ObjectModifiedEvent(survey_session))
        self.assertIsInstance(survey_session.modified, datetime)
        self.assertEqual(survey_session.last_modifier, account)



from plone.formwidget.namedfile.widget import NamedImageWidget
from z3c.form.widget import FieldWidget


class LogoWidget(NamedImageWidget):
    pass


def LogoFieldWidget(field, request):
    return FieldWidget(field, LogoWidget(request))



"""
Sector Container
----------------

A Sector Container provides the overview of all countries.

https://admin.oiraproject.eu/sectors
"""

from euphorie.content.behaviour.richdescription import IRichDescription
from plone.app.dexterity.behaviors.metadata import IBasic
from plone.dexterity.content import Container
from plone.supermodel import model
from zope.interface import implementer


class ISectorContainer(model.Schema, IRichDescription, IBasic):
    """Container for all sectors."""


@implementer(ISectorContainer)
class SectorContainer(Container):
    def _canCopy(self, op=0):
        """Tell Zope2 that this object can not be copied."""
        return False



from euphorie.client import model
from ftw.upgrade import UpgradeStep


class SetThePublishedSessionsToLocked(UpgradeStep):
    """Set the published sessions to locked."""

    def __call__(self):
        sasession = model.Session()
        for session in sasession.query(model.SurveySession).filter(
            model.SurveySession.published != None  # noqa: E711
        ):
            session_event = model.SessionEvent(
                time=session.published,
                account_id=session.last_publisher_id,
                session_id=session.id,
                action="lock_set",
            )
            sasession.add(session_event)

            # Clean up the old publication data
            session.published = None
            session.last_publisher_id = None



from ftw.upgrade import UpgradeStep
from plone.registry.interfaces import IRegistry
from zope.component import getUtility


class RemoveWkhtmltopdfSettings(UpgradeStep):
    """Remove wkhtmltopdf settings."""

    def __call__(self):
        registry = getUtility(IRegistry)
        if "euphorie.wkhtmltopdf.options" in registry.records:
            del registry.records["euphorie.wkhtmltopdf.options"]



from euphorie.testing import EuphorieIntegrationTestCase
from zope.component import getMultiAdapter


class TestSitemenu(EuphorieIntegrationTestCase):
    def test_sitemenu_items_sectors_overview(self):
        """Test, if the site menu contains the expected items."""
        self.loginAsPortalOwner()

        sitemenu = getMultiAdapter((self.portal.sectors, self.request), name="sitemenu")

        menu = sitemenu.actions

        self.assertEqual(menu["title"], "menu_actions")
        self.assertEqual(len(menu["children"]), 2)
        self.assertEqual(menu["children"][0]["title"], "menu_add_new")
        self.assertEqual(menu["children"][1]["title"], "menu_organise")

    def test_sitemenu_items_country(self):
        """Test, if the site menu contains the expected items including the
        country menu on an ICountry context."""
        self.loginAsPortalOwner()

        sitemenu = getMultiAdapter(
            (self.portal.sectors.nl, self.request), name="sitemenu"
        )

        menu = sitemenu.actions

        self.assertEqual(menu["title"], "menu_actions")
        self.assertEqual(len(menu["children"]), 3)
        self.assertEqual(menu["children"][0]["title"], "menu_add_new")
        self.assertEqual(menu["children"][1]["title"], "menu_organise")
        self.assertEqual(menu["children"][2]["title"], "menu_admin")
        self.assertEqual(len(menu["children"][2]["children"]), 1)
        self.assertEqual(
            menu["children"][2]["children"][0]["title"], "menu_country_tools"
        )

    def test_country_menu_availability(self):
        """Test, if the country menu is available in different contexts."""
        sitemenu = getMultiAdapter(
            (self.portal.sectors.nl, self.request), name="sitemenu"
        )
        self.assertIsNotNone(sitemenu.menu_country_tools())

        sitemenu = getMultiAdapter((self.portal.sectors, self.request), name="sitemenu")
        self.assertIsNone(sitemenu.menu_country_tools())

        sitemenu = getMultiAdapter((self.portal, self.request), name="sitemenu")
        self.assertIsNone(sitemenu.menu_country_tools())



from euphorie.client.tests.utils import addSurvey
from euphorie.client.tests.utils import registerUserInClient
from euphorie.testing import EuphorieFunctionalTestCase


class ReportTests(EuphorieFunctionalTestCase):
    def testInvalidDateDoesNotBreakRendering(self):
        from euphorie.client import model
        from euphorie.content.tests.utils import BASIC_SURVEY
        from z3c.saconfig import Session

        import datetime

        # Test for http://code.simplon.biz/tracker/tno-euphorie/ticket/150
        self.loginAsPortalOwner()
        addSurvey(self.portal, BASIC_SURVEY)
        survey_url = self.portal.client.nl["ict"]["software-development"].absolute_url()
        browser = self.get_browser()
        browser.open(survey_url)
        registerUserInClient(browser)
        # We need to manually open the portlet view as the test browser
        # does not handle JavaScript.
        browser.open(
            self.portal.client["nl"].absolute_url() + "/@@portlet-available-tools"
        )
        # Create a new survey session
        browser.getControl(name="survey").value = ["ict/software-development"]
        browser.getForm(action="new-session").submit()
        browser.getControl(name="form.widgets.title").value = "Sessiøn".encode()  # noqa
        # Start the survey
        browser.getControl(name="form.button.submit").click()
        session_url = browser.url.replace("/@@identification", "")
        # Update the risk
        risk = Session.query(model.Risk).first()
        risk.identification = "no"
        risk.action_plans.append(
            model.ActionPlan(
                action_plan="Do something awesome",
                planning_start=datetime.date(1, 2, 3),
            )
        )
        # Render the report
        browser.handleErrors = False
        browser.open("%s/@@report_view" % session_url)
        # No errors = success

    def testCountryDefaultsToCurrentCountry(self):
        from euphorie.content.tests.utils import BASIC_SURVEY

        self.loginAsPortalOwner()
        addSurvey(self.portal, BASIC_SURVEY)
        survey_url = self.portal.client.nl["ict"][
            "software-development"
        ].absolute_url()  # noqa: E501
        browser = self.get_browser()
        browser.open(survey_url)
        registerUserInClient(browser)
        # We need to manually open the portlet view as the test browser
        # does not handle JavaScript.
        browser.open(
            self.portal.client["nl"].absolute_url() + "/@@portlet-available-tools"
        )
        # Create a new survey session
        browser.getControl(name="survey").value = ["ict/software-development"]
        browser.getForm(action="new-session").submit()
        browser.getControl(name="form.widgets.title").value = "Sessiøn".encode()  # noqa
        # Start the survey
        browser.getControl(name="form.button.submit").click()
        session_url = browser.url.replace("/@@identification", "")
        # Check the company data
        browser.open("%s/@@report_company" % session_url)
        self.assertEqual(browser.getControl(name="form.widgets.country").value, ["nl"])

    def testCompanySettingsRoundTrip(self):
        from euphorie.content.tests.utils import BASIC_SURVEY

        self.loginAsPortalOwner()
        addSurvey(self.portal, BASIC_SURVEY)
        browser = self.get_browser()
        survey_url = self.portal.client.nl["ict"]["software-development"].absolute_url()
        browser.open(survey_url)
        registerUserInClient(browser)
        # We need to manually open the portlet view as the test browser
        # does not handle JavaScript.
        browser.open(
            self.portal.client["nl"].absolute_url() + "/@@portlet-available-tools"
        )
        # Create a new survey session
        browser.getControl(name="survey").value = ["ict/software-development"]
        browser.getForm(action="new-session").submit()
        browser.getControl(name="form.widgets.title").value = "Sessiøn".encode()  # noqa
        # Start the survey
        browser.getControl(name="form.button.submit").click()
        session_url = browser.url.replace("/@@identification", "")
        # Enter some company data
        browser.open("%s/@@report_company" % session_url)
        browser.getControl(name="form.widgets.country").value = ["be"]
        browser.getControl(name="form.widgets.employees").value = ["50-249"]
        browser.getControl(name="form.widgets.conductor").value = ["staff"]
        browser.getControl(name="form.widgets.referer").value = ["trade-union"]
        browser.getControl(name="form.widgets.workers_participated").value = ["True"]
        browser.getControl(name="form.buttons.next").click()
        # Make sure all fields validated
        self.assertEqual(browser.url, "%s/@@report_view" % session_url)
        # Verify entered data
        browser.open("%s/@@report_company" % session_url)
        self.assertEqual(browser.getControl(name="form.widgets.country").value, ["be"])
        self.assertEqual(
            browser.getControl(name="form.widgets.employees").value, ["50-249"]
        )
        self.assertEqual(
            browser.getControl(name="form.widgets.conductor").value, ["staff"]
        )
        self.assertEqual(
            browser.getControl(name="form.widgets.referer").value, ["trade-union"]
        )
        self.assertEqual(
            browser.getControl(name="form.widgets.workers_participated").value, ["True"]
        )
